/*
 * Copyright 2008 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
#include <common.h>
#include <linux/types.h>
#include <command.h>

extern int do_saveenv (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);
extern int do_reset (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);
extern void flash_chip_protect(int vpen);
extern __u32 xdcam_get_dipsw(void);
extern int xdcam_jig_boot(void);
extern int xdcam_mb_req_recovery(void);
extern void ibconfig_read(char *data);

#define BOOT_MODE_BOARD    0x00000001
#define BOOT_MODE_JIG      0x00000002        
#define BOOT_MODE_NORMAL   0x00000004
#define BOOT_MODE_RECOVERY 0x00000008

#define BOOT_MODE_BOARD_NORMAL (BOOT_MODE_BOARD | BOOT_MODE_NORMAL)
#define BOOT_MODE_BOARD_RECOVERY (BOOT_MODE_BOARD | BOOT_MODE_RECOVERY)
#define BOOT_MODE_JIG_NORMAL (BOOT_MODE_JIG | BOOT_MODE_NORMAL)
#define BOOT_MODE_JIG_RECOVERY (BOOT_MODE_JIG | BOOT_MODE_RECOVERY)

#define DBOOT_CMD_IDX 0
#define DBOOT_ARG_IDX 1
#define NUM_DBOOT_ENV 2
#define MAX_DBOOT_ENV_LEN 256
static char boot_env_val[NUM_DBOOT_ENV][MAX_DBOOT_ENV_LEN];

void
dualboot_ctrl(void)
{
    unsigned boot_mode;
    int i;
    char *boot_env_str[NUM_DBOOT_ENV];
    char *s;
    char ibconfig_data[CONFIG_IBCONFIG_DATA_SIZE];

            
    /* DIP-SW > instantboot config > environment value > master board control */
    switch ((~xdcam_get_dipsw()) & CONFIG_DIPSW_BOOTSEL_MASK) {
    case CONFIG_DIPSW_NORMAL:
	/* forced normal */
	printf("**** dipsw normal ****\n");
	boot_mode = BOOT_MODE_NORMAL;
        break;
    case CONFIG_DIPSW_RECOVERY:
	/* forced recovery */
	printf("**** dipsw recovery ****\n");
        boot_mode = BOOT_MODE_RECOVERY;
        break;
    default:
        ibconfig_read(ibconfig_data);
        if (  strcmp(ibconfig_data,CONFIG_IBCONFIG_DATA) != 0 )
        { /* entry into recovery mode if normal instant image is damaged */
          boot_mode = BOOT_MODE_RECOVERY;
          break;
        }
        else
        {
          /* check environment value */
          s = getenv("bootselect");
          if ((s != NULL) && (strcmp(s, "normal") == 0)) {
	      printf("**** env-bootselect normal ****\n");
	      boot_mode = BOOT_MODE_NORMAL;
              break;
          } else if ((s != NULL ) && (strcmp(s, "recovery") == 0)) {
	      printf("**** env-bootselect recovery\n");
              boot_mode = BOOT_MODE_RECOVERY;
              break;
          } else {
	      /*  check master board control */
	      if (xdcam_mb_req_recovery()) {
		  printf("**** uITRON-control recovery\n");
		  boot_mode = BOOT_MODE_RECOVERY;
	      } else {
		  printf("**** uITRON-control normal\n");
		  boot_mode = BOOT_MODE_NORMAL;
	      }
              break;
          }
       }
    }
    
    /* Check boot from on-board flash memory or JIG flash memory */
    boot_mode |= (xdcam_jig_boot()) ? BOOT_MODE_JIG : BOOT_MODE_BOARD;

    switch (boot_mode) {
    case BOOT_MODE_BOARD_NORMAL:
        printf("**** on-board flash boot / normal mode ****\n");
        boot_env_str[DBOOT_CMD_IDX] = "bootcmd";
        boot_env_str[DBOOT_ARG_IDX] = "bootargs";
	break;
    case BOOT_MODE_BOARD_RECOVERY:
        printf("**** on-board flash boot / recovery mode ****\n");
        boot_env_str[DBOOT_CMD_IDX] = "bootcmdr";
        boot_env_str[DBOOT_ARG_IDX] = "bootargsr";
	break;
    case BOOT_MODE_JIG_NORMAL:
        printf("**** jig flash boot / normal mode ****\n");
        boot_env_str[DBOOT_CMD_IDX] = "bootcmdj";
        boot_env_str[DBOOT_ARG_IDX] = "bootargsj";
	break;
    case BOOT_MODE_JIG_RECOVERY:
        printf("**** jig flash boot / recovery mode ****\n");
        boot_env_str[DBOOT_CMD_IDX] = "bootcmdjr";
        boot_env_str[DBOOT_ARG_IDX] = "bootargsjr";
	break;
    default:
	printf("**** Error. boot-mode cannot detect. ****\n");
	printf("bootmode %08X\n", boot_mode);
        boot_env_str[DBOOT_CMD_IDX] = "bootcmd";
        boot_env_str[DBOOT_ARG_IDX] = "bootargs";
    }

    /* Copy env strings */
    for (i = 0; i < NUM_DBOOT_ENV; i++) {
	if ((s = getenv (boot_env_str[i])) == NULL) {
	    printf("%s is not setting.\n", boot_env_str[i]);
	} else {
	    strncpy(&(boot_env_val[i][0]), s, MAX_DBOOT_ENV_LEN); 
	}
    }
}

char *
get_boot_env_val(int idx)
{
    return &(boot_env_val[idx][0]);
}

char *
set_boot_env_val(int idx, char *str)
{
    return strncpy(&(boot_env_val[idx][0]), str, MAX_DBOOT_ENV_LEN);
}

int
switch_to_recovery(void)
{
    char *s;

    if ((s = getenv("bootselect")) == NULL)
	return 0;

    if (strcmp(s, "auto") != 0)
    	return 0;

    setenv("bootselect", "recovery");
#if defined(CONFIG_FLASH_WP_CTRL)
    flash_chip_protect(0);
    do_saveenv (NULL, 0, 0, NULL);
    flash_chip_protect(1);
#else
    do_saveenv (NULL, 0, 0, NULL);
#endif

    do_reset (NULL, 0, 0, NULL);
    
    return 0;
}

/* end of <dual-boot.c> */
