/*
 * (C) Copyright 2003 Motorola Inc.
 *  Xianghua Xiao,(X.Xiao@motorola.com)
 *
 * (C) Copyright 2000, 2001
 *  Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2001, Stuart Hughes, Lineo Inc, stuarth@lineo.com
 * Add support the Sharp chips on the mpc8260ads.
 * I started with board/ip860/flash.c and made changes I found in
 * the MTD project by David Schleef.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>

// #define DEBUG

#if !defined(CFG_NO_FLASH)

flash_info_t	flash_info[CFG_MAX_FLASH_BANKS]; /* info for FLASH chips	*/

#if defined(CFG_ENV_IS_IN_FLASH)
# ifndef  CFG_ENV_ADDR
#  define CFG_ENV_ADDR	(CFG_FLASH_BASE + CFG_ENV_OFFSET)
# endif
# ifndef  CFG_ENV_SIZE
#  define CFG_ENV_SIZE	CFG_ENV_SECT_SIZE
# endif
# ifndef  CFG_ENV_SECT_SIZE
#  define CFG_ENV_SECT_SIZE  CFG_ENV_SIZE
# endif
#endif


/*-----------------------------------------------------------------------
 * Functions
 */
unsigned long flash_init(void);
int flash_clear_all_lock_bit(void);
static ulong flash_get_size (vu_long *addr, flash_info_t *info);
static int write_word (flash_info_t *info, ulong dest, ulong data);
static int clear_block_lock_bit(flash_info_t *info, vu_long * addr);
/*-----------------------------------------------------------------------
 */

extern int xdcam_jig_rom_on(void);
extern int xdcam_jig_exist(void);
extern int xdcam_jig_boot(void);
extern vu_long *flash_base[][CFG_MAX_FLASH_BANKS];

static int no_flash_bank;

unsigned long flash_init (void)
{
    unsigned long size;
    int i;
    int boot_mode; /* 0:on board, 1:JIG */

	
    /* Init flash_info */
    for (i = 0; i < CFG_MAX_FLASH_BANKS; ++i) {
	flash_info[i].flash_id = FLASH_UNKNOWN;
	flash_info[i].size = 0;
    }

    /* Check flash board JIG */
    boot_mode = 0;
    if (xdcam_jig_exist()) {
	no_flash_bank = CFG_MAX_FLASH_BANKS;
	if (xdcam_jig_boot()) {
	    printf("**** JIG FLASH boot ****\n");
	    boot_mode = 1;
	}
    } else {
	printf("**** Onboard FLASH boot ****\n");
	no_flash_bank = CFG_MAX_FLASH_BANKS / 2;
    }

    /* Search Flash Memory */
    printf("**** Check Flash ****\n");
    
    for (i = 0; i < no_flash_bank; i++) {
	size = flash_get_size(flash_base[boot_mode][i], &flash_info[i]);

	if (flash_info[i].flash_id == FLASH_UNKNOWN) {
	    printf ("## Unknown FLASH on Bank %d - Size = 0x%08lx = %ld MB\n",
		    i, size, size<<20);
	} else {
	    /* Re-do sizing to get full correct info */
	    // size = flash_get_size(flash_base[boot_mode][i], &flash_info[i]);
	    // flash_info[i].size = size;	
	}
    }

#if CFG_MONITOR_BASE >= CFG_FLASH_BASE
    /* monitor protection ON by default */
    flash_protect(FLAG_PROTECT_SET,
		  CFG_MONITOR_BASE,
		  CFG_MONITOR_BASE - 1 + CFG_MONITOR_LEN,
		  &flash_info[(CFG_MAX_FLASH_BANKS >> (1 - boot_mode)) - 1]);
#endif

#ifdef	CFG_ENV_IS_IN_FLASH
    /* ENV protection ON by default */
    flash_protect(FLAG_PROTECT_SET,
		  CFG_ENV_ADDR,
		  CFG_ENV_ADDR - 1 + CFG_ENV_SECT_SIZE,
		  &flash_info[(CFG_MAX_FLASH_BANKS >> (1 - boot_mode)) - 1]);
#endif

#ifdef CONFIG_BOARD_UNIQUE_DATA
    /* board unique data protect */
    flash_protect(FLAG_PROTECT_SET,
		  CFG_BUDATA_ADDR,
		  CFG_BUDATA_ADDR - 1 + CFG_BUDATA_SECTOR_SIZE,
		  &flash_info[(CFG_MAX_FLASH_BANKS >> (1 - boot_mode)) - 1]);
#endif

#ifdef CONFIG_IBLOADER_BASE
    /* Instant boot loader protect */
    flash_protect(FLAG_PROTECT_SET,
		  CONFIG_IBLOADER_BASE,
		  CONFIG_IBLOADER_BASE - 1 + CONFIG_IBLOADER_SIZE,
		  &flash_info[(CFG_MAX_FLASH_BANKS >> (1 - boot_mode)) - 1]);
#endif

#ifdef CONFIG_IBCONFIG_BASE
    /* Instant boot loader protect */
    flash_protect(FLAG_PROTECT_SET,
		  CONFIG_IBCONFIG_BASE,
		  CONFIG_IBCONFIG_BASE - 1 + CONFIG_IBCONFIG_SIZE,
		  &flash_info[(CFG_MAX_FLASH_BANKS >> (1 - boot_mode)) - 1]);
#endif


    /* Calc all flash size */
    size = 0;
    for (i = 0; i < no_flash_bank; i++)
	size += flash_info[i].size;

    return size;
}

/*-----------------------------------------------------------------------
 * Clear all block-lock-bit
 * 0     - success
 * not 0 - error
 */
int
flash_clear_all_lock_bit(void)
{
    int i, j;
    int ret;

    for (i = 0; i <  no_flash_bank; i++) {
	for (j = 0; j < flash_info[i].sector_count; j++) {
	    if ((ret = clear_block_lock_bit(flash_info+i, 
					    (vu_long *)flash_info[i].start[j])) != 0)
		return ret;
	}
    }
    
    return 0;
}


/*-----------------------------------------------------------------------
 */
void flash_print_info  (flash_info_t *info)
{
	int i;

	if (info->flash_id == FLASH_UNKNOWN) {
		printf ("missing or unknown FLASH type\n");
		return;
	}

	switch (info->flash_id & FLASH_VENDMASK) {
	case FLASH_MAN_INTEL:	printf ("Intel ");		break;
	case FLASH_MAN_SHARP:   printf ("Sharp ");		break;
	default:		printf ("Unknown Vendor ");	break;
	}

	switch (info->flash_id & FLASH_TYPEMASK) {
	case FLASH_28F016SV:	printf ("28F016SV (16 Mbit, 32 x 64k)\n");
				break;
	case FLASH_28F160S3:	printf ("28F160S3 (16 Mbit, 32 x 512K)\n");
				break;
	case FLASH_28F320S3:	printf ("28F320S3 (32 Mbit, 64 x 512K)\n");
				break;
	case FLASH_LH28F016SCT: printf ("28F016SC (16 Mbit, 32 x 64K)\n");
				break;
	case FLASH_28F640J3A:   printf ("28F640J3A (64 Mbit, 64 x 128K)\n");
				break;
	case FLASH_28F128J3A:   printf ("28F128J3A (128 Mbit, 128 x 128K)\n");
				break;
	case FLASH_28F256J3C:   printf ("28F256J3C (256 Mbit, 256 x 128K)\n");
				break;
	case FLASH_28F256P30T:  printf ("28F256P30T (256 Mbit, 255 x 128K + 4 x 32K)\n");
				break;
	default:		printf ("Unknown Chip Type\n");
				break;
	}

	printf ("  Size: %ld MB in %d Sectors\n",
		info->size >> 20, info->sector_count);

	printf ("  Sector Start Addresses:");
	for (i=0; i<info->sector_count; ++i) {
		if ((i % 5) == 0)
			printf ("\n   ");
		printf (" %08lX%s",
			info->start[i],
			info->protect[i] ? " (RO)" : "     "
		);
	}
	printf ("\n");
}

static ulong flash_get_size (vu_long *addr, flash_info_t *info)	/* only deal with 16 bit and 32 bit port width, 16bit chip */
{
	short i;
	ulong value,va,vb,vc,vd;
	ulong base = (ulong)addr;
	ulong sector_offset;

#ifdef DEBUG
	printf("Check flash at 0x%08x\n",(uint)addr);
#endif
	/* Write "Intelligent Identifier" command: read Manufacturer ID */
	*addr = 0x90909090;
	udelay(20);
	asm("sync");

#ifndef CFG_FLASH_CFI
	printf("Not define CFG_FLASH_CFI\n");
	return (0);
#else
	value = addr[0];
	va=(value & 0xFF000000)>>24;
	vb=(value & 0x00FF0000)>>16;
	vc=(value & 0x0000FF00)>>8;
	vd=(value & 0x000000FF);
	if ((va==0) && (vb==0)) {
		printf("cannot identify Flash\n");
		info->flash_id = FLASH_UNKNOWN;
		info->sector_count = 0;
		info->size = 0;
		return (0);			/* no or unknown flash	*/
	}
	else if ((va==0) && (vb!=0)) {
		info->big_endian = 1;
		info->chipwidth = FLASH_CFI_BY16;
		if(vb == vd) info->portwidth = FLASH_CFI_32BIT;
		else info->portwidth = FLASH_CFI_16BIT;
	}
	else if ((va!=0) && (vb==0)) {
		info->big_endian = 0;
		info->chipwidth = FLASH_CFI_BY16;
		if(va == vc) info->portwidth = FLASH_CFI_32BIT;
		else info->portwidth = FLASH_CFI_16BIT;
	}
	else if ((va!=0) && (vb!=0)) {
		info->big_endian = 1;		/* no meaning for 8bit chip */
		info->chipwidth = FLASH_CFI_BY8;
		if(va == vb) info->portwidth = FLASH_CFI_16BIT;
		else info->portwidth = FLASH_CFI_8BIT;
	}
#ifdef DEBUG
	switch (info->portwidth) {
		case FLASH_CFI_8BIT:
			printf("port width is 8 bit.\n");
			break;
		case FLASH_CFI_16BIT:
			printf("port width is 16 bit, ");
			break;
		case FLASH_CFI_32BIT:
			printf("port width is 32 bit, ");
			break;
	}
	switch (info->chipwidth) {
		case FLASH_CFI_BY16:
			printf("chip width is 16 bit, ");
			switch (info->big_endian) {
				case 0:
					printf("Little Endian.\n");
					break;
				case 1:
					printf("Big Endian.\n");
					break;
			}
			break;
	}
#endif	
#endif		/*#ifdef CFG_FLASH_CFI*/

	if (info->big_endian==0) value = (addr[0] & 0xFF000000) >>8;
	else value = (addr[0] & 0x00FF0000);
#ifdef DEBUG
	printf("manufacturer=0x%x\n",(uint)(value>>16));
#endif
	switch (value) {
	case MT_MANUFACT & 0xFFFF0000:	/* SHARP, MT or => Intel */
	case INTEL_ALT_MANU & 0xFFFF0000:
		info->flash_id = FLASH_MAN_INTEL;
		break;
	default:
		printf("unknown manufacturer: %x\n", (unsigned int)value);
		info->flash_id = FLASH_UNKNOWN;
		info->sector_count = 0;
		info->size = 0;
		return (0);			/* no or unknown flash	*/
	}

	if (info->portwidth==FLASH_CFI_16BIT) {
		switch (info->big_endian) {
			case 0:
				value = (addr[0] & 0x0000FF00)>>8;
				break;
			case 1:
				value = (addr[0] & 0x000000FF);
				break;
		}
	}
	else if (info->portwidth == FLASH_CFI_32BIT) {
		switch (info->big_endian) {
			case 0:
				value = (addr[1] & 0x0000FF00)>>8;
				break;
			case 1:
				value = (addr[1] & 0x000000FF);
				break;
		}
	}

#ifdef DEBUG
	printf("deviceID=0x%x\n",(uint)value);
#endif
	switch (value) {
	case (INTEL_ID_28F016S & 0x0000FFFF):
		info->flash_id += FLASH_28F016SV;
		info->sector_count = 32;
		sector_offset = 0x10000;
		break;				/* => 2 MB		*/

	case (INTEL_ID_28F160S3 & 0x0000FFFF):
		info->flash_id += FLASH_28F160S3;
		info->sector_count = 32;
		sector_offset = 0x10000;
		break;				/* => 2 MB		*/

	case (INTEL_ID_28F320S3 & 0x0000FFFF):
		info->flash_id += FLASH_28F320S3;
		info->sector_count = 64;
		sector_offset = 0x10000;
		break;				/* => 4 MB		*/

	case (INTEL_ID_28F640J3A & 0x0000FFFF):
		info->flash_id += FLASH_28F640J3A;
		info->sector_count = 64;
		sector_offset = 0x20000;
		break;                          /* => 8 MB             */

	case (INTEL_ID_28F128J3A & 0x0000FFFF):
		info->flash_id += FLASH_28F128J3A;
		info->sector_count = 128;
		sector_offset = 0x20000;
		break;                          /* => 16 MB             */

	case (INTEL_ID_28F256J3C & 0x0000FFFF):
		info->flash_id += FLASH_28F256J3C;
		info->sector_count = 256;
		sector_offset = 0x20000;
		break;                          /* => 32 MB             */

	case (INTEL_ID_28F256P30T & 0x000000FF):
	        info->flash_id += FLASH_28F256P30T;
	        info->sector_count = 259;
	        sector_offset = 0x20000;
	        break;                          /* => 32 MB             */

	case SHARP_ID_28F016SCL & 0x0000FFFF:
	case SHARP_ID_28F016SCZ & 0x0000FFFF:
		info->flash_id      = FLASH_MAN_SHARP | FLASH_LH28F016SCT;
		info->sector_count  = 32;
		sector_offset = 0x10000;
		break;				/* => 2 MB		*/

	default:
		info->flash_id = FLASH_UNKNOWN;
		return (0);			/* => no or unknown flash */

	}


	sector_offset = sector_offset * (info->portwidth / info->chipwidth);

	switch (value) {
	case (INTEL_ID_28F256P30T & 0x000000FF):
	    /* set up sector start address table */
	    for (i = 0; i < info->sector_count - 4; i++) {
		info->start[i] = base;
		base += sector_offset;
		/* don't know how to check sector protection */
		info->protect[i] = 0;
	    }
	    for (; i < info->sector_count; i++) {
		info->start[i] = base;
		base += (sector_offset >> 2);
		/* don't know how to check sector protection */
		info->protect[i] = 0;
	    }
	    info->size = base - info->start[0];
	    break;
	default:
	    /* set up sector start address table */
	    for (i = 0; i < info->sector_count; i++) {
		info->start[i] = base;
		base += sector_offset;
		/* don't know how to check sector protection */
		info->protect[i] = 0;
	    }
	    info->size = info->sector_count * sector_offset;
	}

	/*
	 * Prevent writes to uninitialized FLASH.
	 */
	if (info->flash_id != FLASH_UNKNOWN) {
	    addr = (vu_long *)info->start[0];
	    *addr = 0xFFFFFFFF;	/* reset bank to read array mode */
	    asm("sync");
	}

	return (info->size);
}


/*-----------------------------------------------------------------------
 */

int	flash_erase (flash_info_t *info, int s_first, int s_last)
{
	int flag, prot, sect;
	ulong start, now, last, ready, erase_err_status;


	if (info->big_endian == 1) {
		ready = 0x0080;
		erase_err_status = 0x00a0;
	}
	else {
		ready = 0x8000;
		erase_err_status = 0xa000;
	}
	if ((info->portwidth / info->chipwidth)==2) {
		ready += (ready <<16);
		erase_err_status += (erase_err_status <<16);
	}

#ifdef DEBUG
	printf ("\nReady flag is 0x%lx\nErase error flag is 0x%lx", ready, erase_err_status);
#endif

	if ((s_first < 0) || (s_first > s_last)) {
		if (info->flash_id == FLASH_UNKNOWN) {
			printf ("- missing\n");
		} else {
			printf ("- no sectors to erase\n");
		}
		return 1;
	}

	if (    ((info->flash_id & FLASH_VENDMASK) != FLASH_MAN_INTEL)
	     && ((info->flash_id & FLASH_VENDMASK) != FLASH_MAN_SHARP) ) {
		printf ("Can't erase unknown flash type %08lx - aborted\n",
			info->flash_id);
		return 1;
	}

	prot = 0;
	for (sect=s_first; sect<=s_last; ++sect) {
		if (info->protect[sect]) {
			prot++;
		}
	}

	if (prot) {
		printf ("- Warning: %d protected sectors will not be erased!\n",
			prot);
	} else {
		printf ("\n");
	}

#ifdef DEBUG
	printf("\nFlash Erase:\n");
#endif
	/* Make Sure Block Lock Bit is not set. */
	if(clear_block_lock_bit(info, (vu_long *)(info->start[s_first]))){
		return 1;
	}

	/* Start erase on unprotected sectors */
#if defined(DEBUG)
	printf("Begin to erase now,s_first=0x%x s_last=0x%x...\n",s_first,s_last);
#endif
	printf("\n");
	for (sect = s_first; sect<=s_last; sect++) {
		if (info->protect[sect] == 0) {	/* not protected */
			vu_short *addr16 = (vu_short *)(info->start[sect]);
			vu_long *addr   = (vu_long *)(info->start[sect]);
			printf(".");
			switch (info->portwidth) {
				case FLASH_CFI_16BIT:
					asm("sync");
					last = start = get_timer (0);
					/* Disable interrupts which might cause a timeout here */
					flag = disable_interrupts();
					/* Reset Array */
					*addr16 = 0xffff;
					asm("sync");
					/* Clear Status Register */
					*addr16 = 0x5050;
					asm("sync");
					/* Single Block Erase Command */
					*addr16 = 0x2020;
					asm("sync");
					/* Confirm */
					*addr16 = 0xD0D0;
					asm("sync");
					if((info->flash_id & FLASH_TYPEMASK) != FLASH_LH28F016SCT) {
					    /* Resume Command, as per errata update */
					    *addr16 = 0xD0D0;
					    asm("sync");
					}
					/* re-enable interrupts if necessary */
					if (flag)
						enable_interrupts();
					/* wait at least 80us - let's wait 1 ms */
					*addr16 = 0x7070;
					udelay (1000);
					while ((*addr16 & ready) != ready) { 
						if((*addr16 & erase_err_status)== erase_err_status){
							printf("Error in Block Erase - Lock Bit may be set!\n");
							printf("Status Register = 0x%X\n", (uint)*addr16);
							*addr16 = 0xFFFF;	/* reset bank */
							asm("sync");
							return 1;
						}
						if ((now=get_timer(start)) > CFG_FLASH_ERASE_TOUT) {
							printf ("Timeout\n");
							*addr16 = 0xFFFF;	/* reset bank */
							asm("sync");
							return 1;
						}
						/* show that we're waiting */
						if ((now - last) > 1000) {	/* every second */
							putc ('.');
							last = now;
						}
					}
					/* reset to read mode */
					*addr16 = 0xFFFF;
					asm("sync");
					break;
				case FLASH_CFI_32BIT:
					asm("sync");
					last = start = get_timer (0);
					/* Disable interrupts which might cause a timeout here */
					flag = disable_interrupts();
					/* Reset Array */
					*addr = 0xffffffff;
					asm("sync");
					/* Clear Status Register */
					*addr = 0x50505050;
					asm("sync");
					/* Single Block Erase Command */
					*addr = 0x20202020;
					asm("sync");
					/* Confirm */
					*addr = 0xD0D0D0D0;
					asm("sync");
					if((info->flash_id & FLASH_TYPEMASK) != FLASH_LH28F016SCT) {
					    /* Resume Command, as per errata update */
					    *addr = 0xD0D0D0D0;
					    asm("sync");
					}
					/* re-enable interrupts if necessary */
					if (flag)
						enable_interrupts();
					/* wait at least 80us - let's wait 1 ms */
					*addr = 0x70707070;
					udelay (1000);
					while ((*addr & ready) != ready) {
						if((*addr & erase_err_status)==erase_err_status){
							printf("Error in Block Erase - Lock Bit may be set!\n");
							printf("Status Register = 0x%X\n", (uint)*addr);
							*addr = 0xFFFFFFFF;	/* reset bank */
							asm("sync");
							return 1;
						}
						if ((now=get_timer(start)) > CFG_FLASH_ERASE_TOUT) {
							printf ("Timeout\n");
							*addr = 0xFFFFFFFF;	/* reset bank */
							asm("sync");
							return 1;
						}
						/* show that we're waiting */
						if ((now - last) > 1000) {	/* every second */
							putc ('.');
							last = now;
						}
					}
					/* reset to read mode */
					*addr = 0xFFFFFFFF;
					asm("sync");
					break;
			}	/* end switch */
		}		/* end if */

		printf("\rSector No %04d", sect);
	}			/* end for */

	printf ("\nflash erase done\n");
	return 0;
}


/*-----------------------------------------------------------------------
 * Copy memory to flash, returns:
 * 0 - OK
 * 1 - write timeout
 * 2 - Flash not erased
 */

int write_buff (flash_info_t *info, uchar *src, ulong addr, ulong cnt)
{
	ulong cp, wp, data;
	int i, l, rc;

	printf("\n");

	wp = (addr & ~3);	/* get lower word aligned address */

	/*
	 * handle unaligned start bytes
	 */
	if ((l = addr - wp) != 0) {
		data = 0;
		for (i=0, cp=wp; i<l; ++i, ++cp) {
			data = (data << 8) | (*(uchar *)cp);
		}
		for (; i<4 && cnt>0; ++i) {
			data = (data << 8) | *src++;
			--cnt;
			++cp;
		}
		for (; cnt==0 && i<4; ++i, ++cp) {
			data = (data << 8) | (*(uchar *)cp);
		}

		if ((rc = write_word(info, wp, data)) != 0) {
			return (rc);
		}
		wp += 4;
	}

	/*
	 * handle word aligned part
	 */
	while (cnt >= 4) {
		data = 0;
		for (i=0; i<4; ++i) {
			data = (data << 8) | *src++;
		}
		if ((rc = write_word(info, wp, data)) != 0) {
			return (rc);
		}
		wp  += 4;
		cnt -= 4;
	}

	if (cnt == 0) {
		return (0);
	}

	/*
	 * handle unaligned tail bytes
	 */
	data = 0;
	for (i=0, cp=wp; i<4 && cnt>0; ++i, ++cp) {
		data = (data << 8) | *src++;
		--cnt;
	}
	for (; i<4; ++i, ++cp) {
		data = (data << 8) | (*(uchar *)cp);
	}

	return (write_word(info, wp, data));
}


/*-----------------------------------------------------------------------
 * Write a word to Flash, returns:
 * 0 - OK
 * 1 - write timeout
 * 2 - Flash not erased
 */
static int write_word (flash_info_t *info, ulong dest, ulong data)
{
    vu_long *addr = (vu_long *)dest;
    ulong start, csr, ready;
    int flag=0;


    switch (info->portwidth) {
    case FLASH_CFI_32BIT:
	/* Check if Flash is (sufficiently) erased */
	if ((*addr & data) != data) {
	    return (2);
	}
	/* Disable interrupts which might cause a timeout here */
	flag = disable_interrupts();

	if (info->big_endian == 1) {
	    ready = 0x0080;
	}
	else {
	    ready = 0x8000;
	}
	if ((info->portwidth / info->chipwidth)==2) {
	    ready += (ready <<16);
	}
	else {
	    ready = ready << 16;
	}
	/* Write Command */
	*addr = 0x10101010;
	asm("sync");
	/* Write Data */
	*addr = data;
	/* re-enable interrupts if necessary */
	if (flag)
	    enable_interrupts();
	/* data polling for D7 */
	start = get_timer (0);
	flag  = 0;
	while (((csr = *addr) & ready) != ready) {
	    if (get_timer(start) > CFG_FLASH_WRITE_TOUT) {
		flag = 1;
		break;
	    }
	}
	if (csr & 0x40404040) {
	    printf ("CSR indicates write error (%08lx) at %08lx\n", csr, (ulong)addr);
	    flag = 1;
	}
	/* Clear Status Registers Command */
	*addr = 0x50505050;
	asm("sync");
	/* Reset to read array mode */
	*addr = 0xFFFFFFFF;
	asm("sync");
	break;
    case FLASH_CFI_16BIT:
    default:
	printf("error %s %d", __FILE__, __LINE__);
	flag=3;
	break;
    }

    /* print address */
    if ((dest & 0xFFFF) == 0)
	printf("Address:%08x\r", dest);
    else if ((dest & 0xFFFF) == 0xFFFC)
	printf("Address:%08x\r", dest | 0x3);

    return (flag);
}

/*-----------------------------------------------------------------------
 * Clear Block Lock Bit, returns:
 * 0 - OK
 * 1 - Timeout
 */

static int clear_block_lock_bit(flash_info_t * info, vu_long  * addr)
{
	ulong start, now, ready;

	/* Reset Array */
	*addr = 0xffffffff;
	asm("sync");
	/* Clear Status Register */
	*addr = 0x50505050;
	asm("sync");

	*addr = 0x60606060;
	asm("sync");
	*addr = 0xd0d0d0d0;
	asm("sync");

	
	if (info->big_endian == 1) {
		ready = 0x0080;
	}
	else {
		ready = 0x8000;
	}
	if ((info->portwidth / info->chipwidth)==2) {
		ready += (ready <<16);
	}
	else {
		ready = ready << 16;
	}
#ifdef DEBUG
	printf ("%s: Ready flag is 0x%8lx\n", __FUNCTION__, ready);
#endif
	*addr = 0x70707070;	/* read status */
	start = get_timer (0);
	while((*addr & ready) != ready){
		if ((now=get_timer(start)) > CFG_FLASH_ERASE_TOUT) {
			printf ("Timeout on clearing Block Lock Bit\n");
			*addr = 0xFFFFFFFF;	/* reset bank */
			asm("sync");
			return 1;
		}
	}

	*addr = 0xFFFFFFFF;
	return 0;
}

#endif /* !CFG_NO_FLASH */

/*-----------------------------------------------------------------------
 * Support for flash file system (JFFS2)
 *
 * We use custom partition info function because we have to fit the
 * file system image between first sector (containing hard reset
 * configuration word) and the sector containing U-Boot image. Standard
 * partition info function does not allow for last sector specification
 * and assumes that the file system occupies flash bank up to and
 * including bank's last sector.
 */
#if (CONFIG_COMMANDS & CFG_CMD_JFFS2) && defined(CFG_JFFS_CUSTOM_PART)

#ifndef CFG_JFFS2_FIRST_SECTOR
#define CFG_JFFS2_FIRST_SECTOR 0
#endif
#ifndef CFG_JFFS2_FIRST_BANK
#define CFG_JFFS2_FIRST_BANK 0
#endif
#ifndef CFG_JFFS2_NUM_BANKS
#define CFG_JFFS2_NUM_BANKS 1
#endif
#define CFG_JFFS2_LAST_BANK (CFG_JFFS2_FIRST_BANK + CFG_JFFS2_NUM_BANKS - 1)

#include <jffs2/jffs2.h>

static struct part_info partition;

struct part_info *jffs2_part_info(int part_num)
{
	int i;

	if (part_num == 0) {
		if (partition.usr_priv == 0) {
			partition.offset =
				(unsigned char *) flash_info[CFG_JFFS2_FIRST_BANK].start[CFG_JFFS2_FIRST_SECTOR];
			for (i = CFG_JFFS2_FIRST_BANK; i <= CFG_JFFS2_LAST_BANK; i++)
				partition.size += flash_info[i].size;
			partition.size -=
				flash_info[CFG_JFFS2_FIRST_BANK].start[CFG_JFFS2_FIRST_SECTOR] -
				flash_info[CFG_JFFS2_FIRST_BANK].start[0];
#ifdef CFG_JFFS2_LAST_SECTOR
			i = flash_info[CFG_JFFS2_LAST_BANK].sector_count - 1;
			partition.size -=
				flash_info[CFG_JFFS2_LAST_BANK].start[i] -
				flash_info[CFG_JFFS2_LAST_BANK].start[CFG_JFFS2_LAST_SECTOR];
#endif

			partition.usr_priv = (void *)1;
		}
		return &partition;
	}
	return 0;
}

#endif /* JFFS2 */
