/*
 * (C) Copyright 2001-2003
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * Modified during 2001 by
 * Advanced Communications Technologies (Australia) Pty. Ltd.
 * Howard Walker, Tuong Vu-Dinh
 *
 * (C) Copyright 2001, Stuart Hughes, Lineo Inc, stuarth@lineo.com
 * Added support for the 16M dram simm on the 8260ads boards
 *
 * (C) Copyright 2003-2004 Arabella Software Ltd.
 * Yuli Barcohen <yuli@arabellasw.com>
 * Added support for SDRAM DIMMs SPD EEPROM, MII, Ethernet PHY init.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <ioports.h>
#include <mpc8260.h>
#include <asm/m8260_pci.h>
#include <i2c.h>
#include <spd.h>
#include <miiphy.h>
#include <pci.h>
#include <linux/types.h>

void flash_chip_protect(int vpen);
volatile __u32 xdcam_get_dipsw(void);


/*
 * I/O Port configuration table
 *
 * if conf is 1, then that port pin will be configured at boot time
 * according to the five values podr/pdir/ppar/psor/pdat for that entry
 */

#define CFG_FCC1 (CONFIG_ETHER_INDEX == 1)
#define CFG_FCC2 (CONFIG_ETHER_INDEX == 2)
#define CFG_FCC3 (CONFIG_ETHER_INDEX == 3)

const iop_conf_t iop_conf_tab[4][32] = {
    /*
      name | 0 / 1
      -----+----------------------
      conf | disable / enable
      ppar | GPIO / function
      psor | Option 1 / Option 2
      pdir | in  / out
      podr | Regular / Open drain
      pdat | initial DATA
    */

    /* Port A configuration */
    {   /*            conf      ppar psor pdir podr pdat */
        /* PA31 */ { CFG_FCC1,   1,   1,   0,   0,   0 }, /* FCC1 MII COL   */
        /* PA30 */ { CFG_FCC1,   1,   1,   0,   0,   0 }, /* FCC1 MII CRS   */
        /* PA29 */ { CFG_FCC1,   1,   1,   1,   0,   0 }, /* FCC1 MII TX_ER */
        /* PA28 */ { CFG_FCC1,   1,   1,   1,   0,   0 }, /* FCC1 MII TX_EN */
        /* PA27 */ { CFG_FCC1,   1,   1,   0,   0,   0 }, /* FCC1 MII RX_DV */
        /* PA26 */ { CFG_FCC1,   1,   1,   0,   0,   0 }, /* FCC1 MII RX_ER */
        /* PA25 */ { 0,          0,   0,   0,   0,   0 }, /* PA25 */
        /* PA24 */ { 0,          0,   0,   0,   0,   0 }, /* PA24 */
        /* PA23 */ { 0,          0,   0,   0,   0,   0 }, /* PA23 */
        /* PA22 */ { 0,          0,   0,   0,   0,   0 }, /* PA22 */
        /* PA21 */ { CFG_FCC1,   1,   0,   1,   0,   0 }, /* FCC1 MII TxD[3] */
        /* PA20 */ { CFG_FCC1,   1,   0,   1,   0,   0 }, /* FCC1 MII TxD[2] */
        /* PA19 */ { CFG_FCC1,   1,   0,   1,   0,   0 }, /* FCC1 MII TxD[1] */
        /* PA18 */ { CFG_FCC1,   1,   0,   1,   0,   0 }, /* FCC1 MII TxD[0] */
        /* PA17 */ { CFG_FCC1,   1,   0,   0,   0,   0 }, /* FCC1 MII RxD[0] */
        /* PA16 */ { CFG_FCC1,   1,   0,   0,   0,   0 }, /* FCC1 MII RxD[1] */
        /* PA15 */ { CFG_FCC1,   1,   0,   0,   0,   0 }, /* FCC1 MII RxD[2] */
        /* PA14 */ { CFG_FCC1,   1,   0,   0,   0,   0 }, /* FCC1 MII RxD[3] */
        /* PA13 */ { 0,          0,   0,   0,   0,   0 }, /* PA13 */
        /* PA12 */ { 0,          0,   0,   0,   0,   0 }, /* PA12 */
        /* PA11 */ { 0,          0,   0,   0,   0,   0 }, /* PA11 */
        /* PA10 */ { 0,          0,   0,   0,   0,   0 }, /* PA10 */
        /* PA9  */ { 0,          0,   0,   0,   0,   0 }, /* PA9 */
        /* PA8  */ { 0,          0,   0,   0,   0,   0 }, /* PA8 */
        /* PA7  */ { 0,          0,   0,   1,   0,   0 }, /* PA7 */
        /* PA6  */ { 0,          0,   0,   0,   0,   0 }, /* PA6 */
        /* PA5  */ { 0,          0,   0,   1,   0,   0 }, /* PA5 */
        /* PA4  */ { 0,          0,   0,   1,   0,   0 }, /* PA4 */
        /* PA3  */ { 0,          0,   0,   1,   0,   0 }, /* PA3 */
        /* PA2  */ { 0,          0,   0,   1,   0,   0 }, /* PA2 */
        /* PA1  */ { 0,          0,   0,   0,   0,   0 }, /* PA1 */
        /* PA0  */ { 0,          0,   0,   1,   0,   0 }  /* PA0 */
    },

    /* Port B configuration */
    {   /*            conf      ppar psor pdir podr pdat */
        /* PB31 */ { CFG_FCC2,   1,   0,   1,   0,   0 }, /* FCC2 MII TX_ER */
        /* PB30 */ { CFG_FCC2,   1,   0,   0,   0,   0 }, /* FCC2 MII RX_DV */
        /* PB29 */ { CFG_FCC2,   1,   1,   1,   0,   0 }, /* FCC2 MII TX_EN */
        /* PB28 */ { CFG_FCC2,   1,   0,   0,   0,   0 }, /* FCC2 MII RX_ER */
        /* PB27 */ { CFG_FCC2,   1,   0,   0,   0,   0 }, /* FCC2 MII COL */
        /* PB26 */ { CFG_FCC2,   1,   0,   0,   0,   0 }, /* FCC2 MII CRS */
        /* PB25 */ { CFG_FCC2,   1,   0,   1,   0,   0 }, /* FCC2 MII TxD[3] */
        /* PB24 */ { CFG_FCC2,   1,   0,   1,   0,   0 }, /* FCC2 MII TxD[2] */
        /* PB23 */ { CFG_FCC2,   1,   0,   1,   0,   0 }, /* FCC2 MII TxD[1] */
        /* PB22 */ { CFG_FCC2,   1,   0,   1,   0,   0 }, /* FCC2 MII TxD[0] */
        /* PB21 */ { CFG_FCC2,   1,   0,   0,   0,   0 }, /* FCC2 MII RxD[0] */
        /* PB20 */ { CFG_FCC2,   1,   0,   0,   0,   0 }, /* FCC2 MII RxD[1] */
        /* PB19 */ { CFG_FCC2,   1,   0,   0,   0,   0 }, /* FCC2 MII RxD[2] */
        /* PB18 */ { CFG_FCC2,   1,   0,   0,   0,   0 }, /* FCC2 MII RxD[3] */
        /* PB17 */ { CFG_FCC3,   1,   0,   0,   0,   0 }, /* FCC3:RX_DIV */
        /* PB16 */ { CFG_FCC3,   1,   0,   0,   0,   0 }, /* FCC3:RX_ERR */
        /* PB15 */ { CFG_FCC3,   1,   0,   1,   0,   0 }, /* FCC3:TX_ERR */
        /* PB14 */ { CFG_FCC3,   1,   0,   1,   0,   0 }, /* FCC3:TX_EN */
        /* PB13 */ { CFG_FCC3,   1,   0,   0,   0,   0 }, /* FCC3:COL */
        /* PB12 */ { CFG_FCC3,   1,   0,   0,   0,   0 }, /* FCC3:CRS */
        /* PB11 */ { CFG_FCC3,   1,   0,   0,   0,   0 }, /* FCC3:RXD */
        /* PB10 */ { CFG_FCC3,   1,   0,   0,   0,   0 }, /* FCC3:RXD */
        /* PB9  */ { CFG_FCC3,   1,   0,   0,   0,   0 }, /* FCC3:RXD */
        /* PB8  */ { CFG_FCC3,   1,   0,   0,   0,   0 }, /* FCC3:RXD */
        /* PB7  */ { CFG_FCC3,   1,   0,   1,   0,   0 }, /* FCC3:TXD */
        /* PB6  */ { CFG_FCC3,   1,   0,   1,   0,   0 }, /* FCC3:TXD */
        /* PB5  */ { CFG_FCC3,   1,   0,   1,   0,   0 }, /* FCC3:TXD */
        /* PB4  */ { CFG_FCC3,   1,   0,   1,   0,   0 }, /* FCC3:TXD */
        /* PB3  */ { 0,          0,   0,   0,   0,   0 }, /* pin doesn't exist */
        /* PB2  */ { 0,          0,   0,   0,   0,   0 }, /* pin doesn't exist */
        /* PB1  */ { 0,          0,   0,   0,   0,   0 }, /* pin doesn't exist */
        /* PB0  */ { 0,          0,   0,   0,   0,   0 }  /* pin doesn't exist */
    },

    /* Port C */
    {   /*            conf      ppar psor pdir podr pdat */
        /* PC31 */ { 0,          0,   0,   1,   0,   1 }, /* PC31 */
        /* PC30 */ { 0,          0,   0,   1,   0,   1 }, /* PC30 */
        /* PC29 */ { 1,          0,   0,   1,   0,   1 }, /* PC29 D300 */
        /* PC28 */ { 1,          0,   0,   1,   0,   1 }, /* PC28 D303 */
        /* PC27 */ { 1,          0,   0,   1,   0,   1 }, /* PC27 D301 */
        /* PC26 */ { 1,          0,   0,   1,   0,   1 }, /* PC26 D304 */
        /* PC25 */ { 1,          0,   0,   1,   0,   1 }, /* PC25 D302 */
        /* PC24 */ { 1,          0,   0,   1,   0,   1 }, /* PC24 D305 */
        /* PC23 */ { 0,          0,   0,   0,   0,   0 }, /* PC23 */
        /* PC22 */ { CFG_FCC1,   1,   0,   0,   0,   0 }, /* FCC1 MII Tx Clock (CLK10) */
        /* PC21 */ { CFG_FCC1,   1,   0,   0,   0,   0 }, /* FCC1 MII Rx Clock (CLK11) */
        /* PC20 */ { 1,          0,   0,   1,   0,   0 }, /* PC20 ETHER_RST_X */
        /* PC19 */ { 1,          0,   0,   1,   0,   0 }, /* FETHMDC  */
        /* PC18 */ { 1,          0,   0,   0,   0,   0 }, /* FETHMDIO */
        /* PC17 */ { CFG_FCC2,   1,   0,   0,   0,   0 }, /* FCC2 MII Rx Clock (CLK15) */
        /* PC16 */ { CFG_FCC2,   1,   0,   0,   0,   0 }, /* FCC2 MII Tx Clock (CLK16) */
        /* PC15 */ { 1,          0,   0,   0,   0,   0 }, /* PC15 JIG_EXIST_X */
        /* PC14 */ { 1,          0,   0,   0,   0,   0 }, /* PC14 LNX_JIG_ROM_ON_X */
        /* PC13 */ { 1,          0,   0,   0,   0,   0 }, /* PC13 SHUTDOWN_REQ */
        /* PC12 */ { 1,          0,   0,   0,   0,   0 }, /* PC12 RECOVERY_MODE */
        /* PC11 */ { 1,          0,   0,   1,   0,   0 }, /* PC11 SHUTDOWN_ACK */
        /* PC10 */ { 1,          0,   0,   1,   0,   1 }, /* PC10 SELF_NRESET */
        /* PC9  */ { 1,          0,   0,   1,   0,   1 }, /* PC9  LNX_INT_X */
        /* PC8  */ { 1,          0,   0,   1,   0,   0 }, /* PC8  POWER_OK */
        /* PC7  */ { 1,          0,   0,   0,   0,   1 }, /* PC7 */
        /* PC6  */ { 0,          0,   0,   0,   0,   1 }, /* PC6 */
        /* PC5  */ { 0,          0,   0,   0,   0,   1 }, /* PC5 */
        /* PC4  */ { 0,          0,   0,   0,   0,   1 }, /* PC4 */
        /* PC3  */ { 0,          0,   0,   0,   0,   1 }, /* PC3 */
        /* PC2  */ { 0,          0,   0,   0,   0,   1 }, /* PC2 */
        /* PC1  */ { 0,          0,   0,   0,   0,   0 }, /* PC1 */
        /* PC0  */ { 1,          0,   0,   1,   0,   1 }, /* PC0 */
    },

    /* Port D */
    {   /*            conf ppar psor pdir podr pdat */
        /* PD31 */ {   1,   1,   0,   0,   0,   0   }, /* SCC1 UART RxD */
        /* PD30 */ {   1,   1,   1,   1,   0,   0   }, /* SCC1 UART TxD */
        /* PD29 */ {   0,   0,   0,   0,   0,   0   }, /* PD29 */
        /* PD28 */ {   0,   1,   0,   0,   0,   0   }, /* PD28 */
        /* PD27 */ {   0,   1,   1,   1,   0,   0   }, /* PD27 */
        /* PD26 */ {   0,   0,   0,   1,   0,   0   }, /* PD26 */
        /* PD25 */ {   0,   0,   0,   0,   0,   0   }, /* PD25 S300:1 / CPU_DIPSW_0 /CPLD DIP-SW */
        /* PD24 */ {   0,   0,   0,   0,   0,   0   }, /* PD24 S300:2 / LNX_JIG_ROM_ON_X */
        /* PD23 */ {   0,   0,   0,   0,   0,   0   }, /* PD23 S300:3 / ON: Force normal boot */
        /* PD22 */ {   0,   0,   0,   0,   0,   0   }, /* PD22 S300:4 / ON: Force recovery boot */
        /* PD21 */ {   0,   0,   0,   0,   0,   0   }, /* PD21 S300:5 / ON: Ignore PWR_BTN */
        /* PD20 */ {   0,   0,   0,   0,   0,   0   }, /* PD20 S300:6 */
        /* PD19 */ {   0,   0,   0,   0,   0,   0   }, /* PD19 S300:7 */
        /* PD18 */ {   0,   0,   0,   0,   0,   0   }, /* PD18 S300 8 */
        /* PD17 */ {   0,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXPRTY */
        /* PD16 */ {   0,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXPRTY */
        /* PD15 */ {   0,   1,   1,   0,   1,   0   }, /* I2C SDA */
        /* PD14 */ {   0,   1,   1,   0,   1,   0   }, /* I2C SCL */
        /* PD13 */ {   0,   0,   0,   0,   0,   0   }, /* PD13 */
        /* PD12 */ {   0,   0,   0,   0,   0,   0   }, /* PD12 */
        /* PD11 */ {   0,   0,   0,   0,   0,   0   }, /* PD11 */
        /* PD10 */ {   0,   0,   0,   0,   0,   0   }, /* PD10 */
        /* PD9  */ {   0,   1,   0,   1,   0,   0   }, /* SMC1 TXD */
        /* PD8  */ {   0,   1,   0,   0,   0,   0   }, /* SMC1 RXD */
        /* PD7  */ {   0,   0,   0,   1,   0,   1   }, /* PD7 */
        /* PD6  */ {   0,   0,   0,   1,   0,   1   }, /* PD6 */
        /* PD5  */ {   0,   0,   0,   1,   0,   1   }, /* PD5 */
        /* PD4  */ {   0,   0,   0,   1,   0,   1   }, /* PD4 */
        /* PD3  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
        /* PD2  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
        /* PD1  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
        /* PD0  */ {   0,   0,   0,   0,   0,   0   }  /* pin doesn't exist */
    }
};

/* Flash Memory Base Address Table (Referenced flash.c) */
#ifdef CONFIG_SY355_01
vu_long *flash_base[2][CFG_MAX_FLASH_BANKS] = {
    {
	(vu_long *)CFG_FLASH_1_BASE,
	(vu_long *)CFG_FLASH_0_BASE,
	(vu_long *)CFG_FLASH_1_JIG_BASE,
	(vu_long *)CFG_FLASH_0_JIG_BASE
    },
    {
	(vu_long *)CFG_FLASH_1_JIG_BASE,
	(vu_long *)CFG_FLASH_0_JIG_BASE,
	(vu_long *)CFG_FLASH_1_BASE,
	(vu_long *)CFG_FLASH_0_BASE
    }
};
#else
vu_long *flash_base[2][CFG_MAX_FLASH_BANKS] = {
    {
	(vu_long *)CFG_FLASH_0_BASE,
	(vu_long *)CFG_FLASH_0_JIG_BASE
    },
    {
	(vu_long *)CFG_FLASH_0_JIG_BASE,
	(vu_long *)CFG_FLASH_0_BASE
    }
};
#endif

void
reset_phy (void)
{
    volatile ioport_t *outp;

    if ((xdcam_get_dipsw() & CONFIG_DIPSW_DBG_MODE) == 0) {
	outp = ioport_addr((immap_t *)CFG_IMMR, CONFIG_GPIO_PN); 
	outp->pdat &= ~CONFIG_ETHERNET_RST_X;
	udelay(2);
	outp->pdat |= CONFIG_ETHERNET_RST_X;
    }
}


int
board_early_init_f(void)
{
    volatile immap_t *immap = (immap_t *) CFG_IMMR;

    immap->im_clkrst.car_sccr |= M826X_SCCR_PCI_MODE_EN;
    immap->im_siu_conf.sc_siumcr =
        (immap->im_siu_conf.sc_siumcr & ~SIUMCR_LBPC11)
        | SIUMCR_LBPC01;
    
    return 0;
}


long int
initdram(int board_type)
{
    long int msize = CFG_SDRAM_SIZE;
#ifndef CFG_RAMBOOT
    volatile immap_t *immap = (immap_t *) CFG_IMMR;
    volatile memctl8260_t *memctl = &immap->im_memctl;
    volatile uchar *ramaddr, c = 0xff;
    uint or;
    uint psdmr;
    uint psrt;
    int i;

    immap->im_siu_conf.sc_ppc_acr  = 0x00000002;
    immap->im_siu_conf.sc_ppc_alrh = 0x01267893;
    immap->im_siu_conf.sc_tescr1   = 0x00004000;

    memctl->memc_mptpr = CFG_MPTPR;

    /* Init 60x bus SDRAM */
    or    = CFG_OR4;
    psdmr = CFG_PSDMR;
    psrt  = CFG_PSRT;

    memctl->memc_psrt = psrt;
    memctl->memc_or4 = or;
    memctl->memc_br4 = CFG_SDRAM_BASE | 0x00000041;
    ramaddr = (uchar *) CFG_SDRAM_BASE;

	udelay(1000);	/* It waits for SDRAM to get warm */

    memctl->memc_psdmr = psdmr | 0x28000000;    /* Precharge all banks */
    *ramaddr = c;
    memctl->memc_psdmr = psdmr | 0x08000000;    /* CBR refresh */
    for (i = 0; i < 8; i++)
        *ramaddr = c;

    memctl->memc_psdmr = psdmr | 0x18000000;    /* Mode Register write */
    *ramaddr = c;
    memctl->memc_psdmr = psdmr | 0x40000000;    /* Refresh enable */
    *ramaddr = c;
#endif /* CFG_RAMBOOT */

    /* return total 60x bus SDRAM size */
    return (msize * 1024 * 1024);
}


int
checkboard(void)
{
    puts ("Board: Sony SY-355\n");

    return 0;
}

#if defined(CFG_DRAM_TEST)

static void ppcDcbf(unsigned long addr)
{
	__asm__("dcbf r0, %0\n" 
			: "=&r" (addr)
			);
}

static void ppcDcbi(unsigned long addr)
{
	__asm__("dcbi r0, %0\n" 
			: "=&r" (addr)
			);
}

int
testdram(void)
{
	uint64_t *pstart = (uint64_t *) CFG_MEMTEST_ALL_START;
	uint64_t *pend = (uint64_t *) CFG_MEMTEST_ALL_END;
	volatile uint64_t *p;
	int err_cnt;
	int is_dcache_enable = dcache_status();

	if((xdcam_get_dipsw() & CONFIG_DIPSW_RAMTEST) != 0){
		return 0;
	}

	printf("Testing DRAM from 0x%08x to 0x%08x\n",
	       CFG_MEMTEST_ALL_START,
	       CFG_MEMTEST_ALL_END);
	for (err_cnt = 0, p = pstart; p < pend; p++){
		*p = 0xa5a5a5a5a5a5a5a5ULL;
		if(is_dcache_enable){
			/* cache flush and invalidate */
			ppcDcbf((unsigned long)p);
			ppcDcbi((unsigned long)p);
		}
		if (*p != 0xa5a5a5a5a5a5a5a5ULL) {
			printf ("\nDRAM 0xa5 test fails at: addr=0x%08x, data=0x%08x%08x\n", (uint)p, (uint)((*p) >> 32), (uint)(*p));
			err_cnt++;
			if(err_cnt > 100){
				printf("DRAM test failed for error max\n");
				return 1;
			}
		}
		*p = 0x5a5a5a5a5a5a5a5aULL;
		if(is_dcache_enable){
			/* cache flush and invalidate */
			ppcDcbf((unsigned long)p);
			ppcDcbi((unsigned long)p);
		}
		if (*p != 0x5a5a5a5a5a5a5a5aULL) {
			printf ("\nDRAM 0x5a test fails at: addr=0x%08x, data=0x%08x%08x\n", (uint)p, (uint)((*p) >> 32), (uint)(*p));
			err_cnt++;
			if(err_cnt > 100){
				printf("DRAM test failed for error max\n");
				return 1;
			}
		}
		if(((uint)p & 0x000fffffUL) == 0){
			printf("\r0x%08x/0x%08x", (uint)p - (uint)pstart, (uint)pend - (uint)pstart);
		}
	}
	printf("\r0x%08x/0x%08x\n", (uint)p - (uint)pstart, (uint)pend - (uint)pstart);

	if(err_cnt == 0){
		printf("DRAM test passed.\n");
	}else{
		printf("DRAM test failed err_cnt = %d\n", err_cnt);
		return 1;
	}

	return 0;
}
#endif	/* CFG_DRAM_TEST */

volatile __u32
xdcam_get_dipsw(void)
{
    volatile ioport_t *inp_sw;
    inp_sw = ioport_addr((immap_t *)CFG_IMMR, CONFIG_DIPSW_PN); 

    return inp_sw->pdat;
}

int 
is_debug_mode(void)
{
    return (xdcam_get_dipsw() & CONFIG_DIPSW_DBG_MODE) ? 0: 1;
}

__u32
xdcam_mb_req_recovery(void)
{
    volatile ioport_t *inp;

    inp = ioport_addr((immap_t *)CFG_IMMR, CONFIG_GPIO_PN); 

    return (inp->pdat & CONFIG_RECEV_MODE_X) ? 0 : 1;
}

#ifdef CONFIG_PCI
void
xdcam_wait_power_btn(void)
{
    volatile ioport_t *outp;

    outp = ioport_addr((immap_t *)CFG_IMMR, CONFIG_GPIO_PN); 

#ifndef CONFIG_SONYSY355
    {
	volatile ioport_t *inp = outp;
	/* Wait master board power control */
	if (xdcam_get_dipsw() & CONFIG_DIPSW_PWRCTRL) {
	    printf("Waiting power control from master board\n");
	    while(inp->pdat & CONFIG_PWR_BTN_X);
	}
    }
#endif

    /* POWER_OK LOW->HIGH */
    outp->pdat |= CONFIG_OUTP_POWER_OK;
    printf("Power ON\n");
}


struct pci_controller hose;
extern void pci_mpc8250_init(struct pci_controller *);

void
pci_init_board(void)
{
    xdcam_wait_power_btn();
    
    pci_mpc8250_init(&hose);
}
#endif /* CONFIG_PCI */

/*
  for flash.c
*/
int
xdcam_jig_rom_on(void)
{
    return (xdcam_get_dipsw() & 0x00000080) ? 0 : 1;
}

int
xdcam_jig_exist(void)
{
    volatile ioport_t *port;

    port = ioport_addr((immap_t *)CFG_IMMR, 
                       CONFIG_GPIO_PN);
    
    return (port->pdat & CONFIG_JIG_EXIST_X) ? 0 : 1;
}

int
xdcam_jig_boot(void)
{
    return xdcam_jig_rom_on() && xdcam_jig_exist();
}

/*
  Zero bootdelay enable switch check
*/
#ifdef CONFIG_ZERO_BOOTDELAY_DIPSW
int
is_zero_bootdelay_enable(void)
{
    return (xdcam_get_dipsw() & CONFIG_DIPSW_ZERO_BOOTDELAY) ? 0: 1;
}
#endif /* CONFIG_ZERO_BOOTDELAY_DIPSW */


/*
  flash protect control command (VPEN pin control)
*/
void 
flash_chip_protect(int vpen)
{
    volatile ioport_t *outp;
    outp = ioport_addr((immap_t *)CFG_IMMR, CONFIG_GPIO_PN); 
    extern int flash_clear_all_lock_bit(void);


    if (vpen) {
        outp->pdat |= CONFIG_OUTP_FLASH_PROTECT;
    } else {
        outp->pdat &= ~CONFIG_OUTP_FLASH_PROTECT;       
	flash_clear_all_lock_bit();
    }
}


int 
do_flash_chip_protect(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    if (argc != 2) {
        printf("usage : fcprotect <on | off>\n");
        printf("ex : fcprotect off\n");
        return 1;
    }
    
    if ((strcmp(argv[1], "off") == 0) || (strcmp(argv[1], "OFF") == 0)) {
        flash_chip_protect(0);
        printf("Flash memory chip protect off\n");
    } else     if ((strcmp(argv[1], "on") == 0) || (strcmp(argv[1], "ON") == 0)) {
        flash_chip_protect(1);
        printf("Flash memory chip protect on\n");
    } else {
        printf("Illegal argument\n");
    }
        

    return 0;
}

/*
  register fprotect command
*/
U_BOOT_CMD(
           fcprotect, 2, 0, do_flash_chip_protect,
           "fcprotect - flash memory chip protect control \n",
           "- flash memory chip protect control \n"
           "usage : fcprotect <on|off>\n"
           );


#if (CONFIG_COMMANDS & CFG_CMD_NET) && defined(CONFIG_NET_MULTI) && \
	defined(CONFIG_E1000)

extern int e1000_set_test_mode(unsigned long mode);
extern unsigned long e1000_get_test_mode(void);
extern int e1000_set_jitter_test_clk(unsigned long data);
extern unsigned long e1000_get_jitter_test_clk(void);

/* 
 * e1000 test mode command
 */
int
do_set_e1000_test_mode(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	unsigned long mode;

	if (argc != 2) {
		printf("usage : e1000setmode <mode(hex)>\n"
			   "\t000b(0x00) = Normal Mode.\n"
			   "\t001b(0x01) = Test Mode 1 - Transmit Waveform Test.\n"
			   "\t010b(0x02) = Test Mode 2 - Transmit Jitter Test (MASTER mode).\n"
			   "\t011b(0x03) = Test Mode 3 - Transmit Jitter Test (SLAVE mode).\n"
			   "\t100b(0x04) = Test Mode 4 - Transmit Distortion Test.\n");
	}

	/* set mode */
	mode = simple_strtoul(argv[1], NULL, 16);
	if ((mode < 0) || (4 < mode)) {
		printf("Invalid Argument\n");
		return -1;
	}
	if (e1000_set_test_mode(mode)) {
		printf("set Test mode Error\n");
	}

	return 0;
}

U_BOOT_CMD(
	e1000setmode, 2, 0, do_set_e1000_test_mode,
	"e1000setmode - set Test mode (bit 13:15 in 1000BASE-T Control Register)\n",
	"- set Test mode (bit 13:15 in 1000BASE-T Control Register)\n"
	"usage : e1000setmode <mode(hex)>\n"
	"\t000b(0x00) = Normal Mode.\n"
	"\t001b(0x01) = Test Mode 1 - Transmit Waveform Test.\n"
	"\t010b(0x02) = Test Mode 2 - Transmit Jitter Test (MASTER mode).\n"
	"\t011b(0x03) = Test Mode 3 - Transmit Jitter Test (SLAVE mode).\n"
	"\t100b(0x04) = Test Mode 4 - Transmit Distortion Test.\n"
);

int
do_get_e1000_test_mode(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	unsigned long value;

	/* get mode */
	value  = e1000_get_test_mode();
	if (value < 0) {
		printf("get Test mode Error\n");
	} else {
		printf("e1000 current Test mode : %d\n", value);
	}

	return 0;
}

U_BOOT_CMD(
	e1000getmode, 1, 0, do_get_e1000_test_mode,
	"e1000getmode - get Test mode (bit 13:15 in 1000BASE-T Control Register)\n",
	"- get Test mode (bit 13:15 in 1000BASE-T Control Register)\n"
	"usage : e1000getmode\n"
);

int
do_set_e1000_jitter_test_clk(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int ret = 0;

	if (argc != 2) {
		printf("usage : e1000setjitclk <on|off>\n");
	}

	/* set Jitter Test Clock */
	if ((!strcmp(argv[1], "on")) || (!strcmp(argv[1], "ON"))) {
		ret = e1000_set_jitter_test_clk(1);
	} else if ((!strcmp(argv[1], "off")) || (!strcmp(argv[1], "OFF"))) {
		ret = e1000_set_jitter_test_clk(0);
	} else {
		printf("usage : e1000setjitclk <on|off>\n");
	}

	if (ret != 0) {
		printf("set Jitter Test Clock Error\n");
	}

	return 0;
}

U_BOOT_CMD(
	e1000setjitclk, 2, 0, do_set_e1000_jitter_test_clk,
	"e1000setjitclk - set Jitter Test Clock (bit 15 in PHY Port Control Register)\n",
	"- set Jitter Test Clock (bit 15 in PHY Port Control Register)\n"
	"usage : e1000setjitclk <on|off>\n"
);

int
do_get_e1000_jitter_test_clk(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	unsigned long value;

	/* get Jitter Test Clock */
	value  = e1000_get_jitter_test_clk();
	if (value < 0) {
		printf("get Jitter Test Clock Error\n");
	} else {
		printf("e1000 current Jitter Test Clock : %d\n", value);
	}

	return 0;
}

U_BOOT_CMD(
	e1000getjitclk, 1, 0, do_get_e1000_jitter_test_clk,
	"e1000getjitclk - get Jitter Test Clock (bit 15 in PHY Port Control Register)\n",
	"- get Jitter Test Clock (bit 15 in PHY Port Control Register)\n"
	"usage : e1000getjitclk\n"
);


#endif /* CFG_CMD_NET, CONFIG_NET_MULTI, CONFIG_E1000 */



/*
  BUD(Board Unique Data) support
  (Ether Net MAC Address, Serial e.t.c.)
*/
#ifdef CONFIG_BOARD_UNIQUE_DATA
#include <flash.h>

#define MAC_ADDR_STR_SIZE 18

extern char *macaddr_read(int devno);
extern int macaddr_write(int devno, char *mac_addr);
extern flash_info_t     flash_info[];

/* Board unique data struct */
typedef struct board_unique_data_t__ {
    uchar mac_addr_tbl[CFG_MAX_ETHDEV][MAC_ADDR_STR_SIZE];
    uchar reseved[0];
} *board_unique_data_ptr;


/*
  copy BUD to RAM
*/
int
bud_read(board_unique_data_ptr unique_data)
{
    /* Copy all flash data to buffer */    
    memcpy(unique_data, (void *)CFG_BUDATA_ADDR, CFG_BUDATA_SIZE);

    return 0;
}

/*
  write BUD data on flash memory
*/
int
bud_write(board_unique_data_ptr unique_data)
{
 
    int boot_mode, ret;

    flash_init();

    boot_mode = (xdcam_jig_boot()) ? 1 : 0;

    /* clear protect */
    flash_protect(FLAG_PROTECT_CLEAR,
                  CFG_BUDATA_ADDR,
                  CFG_BUDATA_ADDR + CFG_BUDATA_SECTOR_SIZE - 1,
		  &flash_info[(CFG_MAX_FLASH_BANKS >> (1 - boot_mode)) - 1]);
    
    /* erase flash */
    ret = flash_sect_erase(CFG_BUDATA_ADDR, 
                           CFG_BUDATA_ADDR + CFG_BUDATA_SECTOR_SIZE - 1);

    if (ret == 0) {
        /* write flash */
        ret = write_buff(
	    &flash_info[(CFG_MAX_FLASH_BANKS >> (1 - boot_mode)) - 1],
	    (uchar *)unique_data,
	    CFG_BUDATA_ADDR, CFG_BUDATA_SIZE);

    }

    /* set protect */
    flash_protect(FLAG_PROTECT_SET,
                  CFG_BUDATA_ADDR,
                  CFG_BUDATA_ADDR + CFG_BUDATA_SECTOR_SIZE - 1,
		  &flash_info[(CFG_MAX_FLASH_BANKS >> (1 - boot_mode)) - 1]);
    
    if (ret)
        printf("Frash Write NG.\n");

    return ret;
}

/*
  read ethernet MAC address
*/
char *  
macaddr_read(int devno)
{
    board_unique_data_ptr data;

    /* Alloc buffer */
    data = (board_unique_data_ptr)CFG_BUDATA_ADDR;

    /* Check MAC address data */
    if (data->mac_addr_tbl[devno][MAC_ADDR_STR_SIZE - 1] != '\0')
        return NULL;

    return &(data->mac_addr_tbl[devno][0]);
}

/*
  write ethernet MAC address
*/
int  
macaddr_write(int devno, char *mac_addr)
{
    static char buff[CFG_BUDATA_SIZE];
    board_unique_data_ptr data = (board_unique_data_ptr)buff;
    int ret;

    if (strlen(mac_addr) != (MAC_ADDR_STR_SIZE - 1)) {
        printf("Error : illegal MAC address.\n");
        return 4;
    }

    /* BUD read */
    bud_read(data);

    /* Copy MAC Address to buffer */
    memcpy(&(data->mac_addr_tbl[devno][0]), mac_addr, MAC_ADDR_STR_SIZE);

    /* Write back BUD */
    ret = bud_write(data);
    
    if (ret == 0)
        setenv("ethaddr", mac_addr);
    
    return ret;
}

/*
  macaddr command
*/
int 
do_macaddr (cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
{
    int ret;

    if ((argc != 2) || (strlen(argv[1]) != (MAC_ADDR_STR_SIZE - 1))) {
        printf("usage : macaddr xx:xx:xx:xx:xx:xx\n");
        printf("ex : macaddr 00:11:22:33:44:55\n");
        return 1;
    }
    
    ret = macaddr_write(0, argv[1]);

    if (ret)
        return 1;
    else 
        return 0;
}

/*
  register macaddr command
*/
U_BOOT_CMD(
           macaddr,     2,      0,      do_macaddr,
           "macaddr  - set macaddress \n",
           "macaddr xx:xx:xx:xx:xx:xx\n"
           );

#endif /* CONFIG_BOARD_UNIQUE_DATA */

/* instantboot config support */
#ifdef CONFIG_IBCONFIG_BASE

/* read from the instantboot config of the flash */
void ibconfig_read(char *data)
{
    memcpy(data, (void *)CONFIG_IBCONFIG_BASE, CONFIG_IBCONFIG_DATA_SIZE);
}

/* write to the instantboot config of the flash */
int ibconfig_write(const char *data,unsigned int size)
{
    int boot_mode, ret;

    flash_init();

    boot_mode = (xdcam_jig_boot()) ? 1 : 0;

    /* clear protect */
    flash_protect(FLAG_PROTECT_CLEAR,
                  CONFIG_IBCONFIG_BASE,
                  CONFIG_IBCONFIG_BASE + CONFIG_IBCONFIG_SIZE - 1,
                  &flash_info[(CFG_MAX_FLASH_BANKS >> (1 - boot_mode)) - 1]);

    /* erase flash */
    ret = flash_sect_erase(CONFIG_IBCONFIG_BASE,
                           CONFIG_IBCONFIG_BASE + CONFIG_IBCONFIG_SIZE - 1);

    if (ret == 0) {
        /* write flash */
        ret = write_buff(
            &flash_info[(CFG_MAX_FLASH_BANKS >> (1 - boot_mode)) - 1],
            (uchar *)data,
            CONFIG_IBCONFIG_BASE,size);
    }

    /* set protect */
    flash_protect(FLAG_PROTECT_SET,
                  CONFIG_IBCONFIG_BASE,
                  CONFIG_IBCONFIG_BASE + CONFIG_IBCONFIG_SIZE - 1,
                  &flash_info[(CFG_MAX_FLASH_BANKS >> (1 - boot_mode)) - 1]);

    if (ret)
        printf("Frash Write NG.\n");

    return ret;
}

/*
  ibconfig command
*/
int 
do_ibconfig (cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
{
    int ret = 0;

    if ( argc == 1 )
    {	/* command: ibconfig */
	ret = ibconfig_write(CONFIG_IBCONFIG_DATA,CONFIG_IBCONFIG_DATA_SIZE);
	printf("Set default value %s to ibconfig. \n",CONFIG_IBCONFIG_DATA);
    }
    else if ( argc == 2 )
    {	/* command: ibconfig <value> */
	if ( strlen(argv[1]) > CONFIG_IBCONFIG_SIZE )
	{
		printf("The size of the value is over the ibconfig maximum %d size. \n",CONFIG_IBCONFIG_SIZE);
		ret = 1;
	}
	else
	{
		ret = ibconfig_write(argv[1],strlen(argv[1]));
		printf("Set value %s to ibconfig. \n",argv[1]);
	}
    }
    else
    {
        printf("ibconfig <value> - set ibconfig value. \n");
        printf("If <value> is not assigned,the command will set the default %s value. \n",CONFIG_IBCONFIG_DATA);
	ret = 1;
    }

    return ret;
}

/*
  register ibconfig command
*/
U_BOOT_CMD(
           ibconfig,     2,      0,      do_ibconfig,
           "ibconfig  <value> - set ibconfig value. \n"
           "If <value> is not assigned, the command will set the default 0422 value.\n",
           "<value> - set ibconfig value. \n"
           "If <value> is not assigned, the command will set the default 0422 value.\n"
	   );
#endif /* CONFIG_IBCONFIG_BASE */


/*
  end of sonysy355.c
*/
