/*
 * This file is based on MPC8260ADS.h  U-Boot config files.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 */

#ifndef __CONFIG_H
#define __CONFIG_H

/*
 * High Level Configuration Options
 * (easy to change)
 */
#define CONFIG_MPC8260ADS       1       /* Motorola PQ2 ADS family board */
#define CONFIG_SONYSY355        1
#define CONFIG_DUALBOOT         1
#define CONFIG_K26
#define CONFIG_USE_ICE

//#define CONFIG_SY355_01         1

/* ADS flavours */
#define CFG_8260ADS             1       /* MPC8260ADS */
#define CFG_8266ADS             2       /* MPC8266ADS */
#define CFG_PQ2FADS             3       /* PQ2FADS-ZU or PQ2FADS-VR */
#define CFG_8272ADS             4       /* MPC8272ADS */

#define CONFIG_ADSTYPE          CFG_8272ADS
#define CONFIG_MPC8272          1
#define CONFIG_SILENT_CONSOLE   1

#ifdef CONFIG_SY355_01
#  define CFG_SDRAM_SIZE          128      /* 128Mbyte */
#else
#  define CFG_SDRAM_SIZE          256      /* 256Mbyte */
#endif
#define CONFIG_BOARD_EARLY_INIT_F 1     /* Call board_early_init_f      */

/* allow serial and ethaddr to be overwritten */
#define CONFIG_ENV_OVERWRITE


#ifdef CONFIG_SY355_01
   /* CS0-3 : flash-memory */
#  define CFG_FLASH_BRx           0x00001801
#  define CFG_FLASH_ORx           0xFE000836
#  define CFG_FLASH_0_BASE        0xFE000000
#  define CFG_FLASH_1_BASE        0xFC000000
#  define CFG_FLASH_0_JIG_BASE    0xFA000000
#  define CFG_FLASH_1_JIG_BASE    0xF8000000
#  define CFG_BR0_PRELIM          (CFG_FLASH_0_BASE | CFG_FLASH_BRx)      /* Board 0 */
#  define CFG_BR1_PRELIM          (CFG_FLASH_1_BASE | CFG_FLASH_BRx)      /* Board 1 */
#  define CFG_BR2_PRELIM          (CFG_FLASH_0_JIG_BASE | CFG_FLASH_BRx)  /* JIG 0 */
#  define CFG_BR3_PRELIM          (CFG_FLASH_1_JIG_BASE | CFG_FLASH_BRx)  /* JIG 1 */
#  define CFG_OR0_PRELIM          CFG_FLASH_ORx /* Board 0 */
#  define CFG_OR1_PRELIM          CFG_FLASH_ORx /* Board 1 */
#  define CFG_OR2_PRELIM          CFG_FLASH_ORx /* JIG 0 */
#  define CFG_OR3_PRELIM          CFG_FLASH_ORx /* JIG 1 */
#  define CFG_FLASH_BASE          CFG_FLASH_1_BASE
#else
   /* CS0-1 : flash-memory */
#  define CFG_FLASH_BRx           0x00001801
#  define CFG_FLASH_ORx           0xFC000854
#  define CFG_FLASH_0_BASE        0xFC000000
#  define CFG_FLASH_0_JIG_BASE    0xF8000000
#  define CFG_FLASH_BASE          CFG_FLASH_0_BASE
#  define CFG_BR0_PRELIM          (CFG_FLASH_0_BASE | CFG_FLASH_BRx)      /* Board 0 */
#  define CFG_BR1_PRELIM          (CFG_FLASH_0_JIG_BASE | CFG_FLASH_BRx)  /* JIG 0 */
#  define CFG_OR0_PRELIM          CFG_FLASH_ORx /* Board 0 */
#  define CFG_OR1_PRELIM          CFG_FLASH_ORx /* JIG 0   */
#endif /* CONFIG_SY355_01 */

/* CS4 : SDRAM */
#ifdef CONFIG_SY355_01
#  define CFG_OR4                 0xF8002B00 /* page based */
#  define CFG_PSDMR               0x832EA462 /* page based */
#  define CFG_PSRT                0x12
#  define CFG_MPTPR               0x2800
#else
#  define CFG_OR4                 0xF0002900 /* page based */
#  define CFG_PSDMR               0x8432A462 /* page based */
#  define CFG_PSRT                0x12
#  define CFG_MPTPR               0x2800
#endif

#define CFG_RESET_ADDRESS       0x08800000
#define CFG_RMR                 RMR_CSRE
#define CFG_TMCNTSC             (TMCNTSC_SEC|TMCNTSC_ALR|TMCNTSC_TCF|TMCNTSC_TCE)
#define CFG_PISCR               (PISCR_PS|PISCR_PTF|PISCR_PTE)
#define CFG_RCCR                0

/* CS5 : JIG GPIO */
#define CFG_BR5_PRELIM          (0xF0108000 | CFG_FLASH_BRx)
#define CFG_OR5_PRELIM          0xFFFF8036

/* CS6 : CPLD */
#define CFG_BR6_PRELIM          (0xF0100000 | CFG_FLASH_BRx)
#define CFG_OR6_PRELIM          0xFFFF8036


/* #define CONFIG_BOOT_16          1 */

/*
 * select serial console configuration
 *
 * if either CONFIG_CONS_ON_SMC or CONFIG_CONS_ON_SCC is selected, then
 * CONFIG_CONS_INDEX must be set to the channel number (1-2 for SMC, 1-4
 * for SCC).
 *
 * if CONFIG_CONS_NONE is defined, then the serial console routines must
 * defined elsewhere (for example, on the cogent platform, there are serial
 * ports on the motherboard which are used for the serial console - see
 * cogent/cma101/serial.[ch]).
 */
#undef  CONFIG_CONS_ON_SMC              /* define if console on SMC */
#define CONFIG_CONS_ON_SCC              /* define if console on SCC */
#undef  CONFIG_CONS_NONE                /* define if console on something else */
#define CONFIG_CONS_INDEX       1       /* which serial channel for console */

/*
 * select ethernet configuration
 *
 * if either CONFIG_ETHER_ON_SCC or CONFIG_ETHER_ON_FCC is selected, then
 * CONFIG_ETHER_INDEX must be set to the channel number (1-4 for SCC, 1-3
 * for FCC)
 *
 * if CONFIG_ETHER_NONE is defined, then either the ethernet routines must be
 * defined elsewhere (as for the console), or CFG_CMD_NET must be removed
 * from CONFIG_COMMANDS to remove support for networking.
 */
#define CONFIG_ETHADDR   00:11:22:33:44:55

#undef CONFIG_E1000                    /* define if 82541PI (1000BASE-T) */
#undef CONFIG_EEPRO100                 /* define if intel 82551 (100BASE-TX) */
#define CONFIG_ETHER_ON_FCC            /* define if ether on fcc   */
#undef CONFIG_ETHER_ON_SCC             /* define if ether on SCC   */
#define CONFIG_ETHER_INDEX  1          /* which SCC/FCC channel for ethernet */
#undef CONFIG_ETHER_NONE               /* define if ether on something else */


#if defined(CONFIG_E1000) || defined(CONFIG_EEPRO100)
#  define CONFIG_NET_MULTI
#endif

#if defined(CONFIG_ETHER_ON_SCC) || defined(CONFIG_ETHER_ON_FCC)
#  if CONFIG_ETHER_INDEX == 1
#    define CFG_PHY_ADDR      0
#    define CFG_CMXFCR_VALUE  (CMXFCR_RF1CS_CLK11 | CMXFCR_TF1CS_CLK10)
#    define CFG_CMXFCR_MASK   (CMXFCR_FC1 | CMXFCR_RF1CS_MSK | CMXFCR_TF1CS_MSK)
#  elif CONFIG_ETHER_INDEX == 2
#    define CFG_PHY_ADDR      3
#    define CFG_CMXFCR_VALUE  (CMXFCR_RF2CS_CLK15 | CMXFCR_TF2CS_CLK16)
#    define CFG_CMXFCR_MASK   (CMXFCR_FC2 | CMXFCR_RF2CS_MSK | CMXFCR_TF2CS_MSK)
#  endif      /* CONFIG_ETHER_INDEX */
#  define CFG_CPMFCR_RAMTYPE  0               /* BDs and buffers on 60x bus */
#  define CFG_FCC_PSMR                (FCC_PSMR_FDE | FCC_PSMR_LPB)  /* Full duplex */
#  define CONFIG_MII                  /* MII PHY management           */
#  define CONFIG_BITBANGMII           /* bit-bang MII PHY management  */

  /*
   * GPIO pins used for bit-banged MII communications
   */
#  define MDIO_PORT     2               /* Port C */
#  define CFG_MDIO_PIN  0x00002000      /* PC18 */
#  define CFG_MDC_PIN   0x00001000      /* PC19 */

#  define MDIO_ACTIVE (iop->pdir |=  CFG_MDIO_PIN)
#  define MDIO_TRISTATE       (iop->pdir &= ~CFG_MDIO_PIN)
#  define MDIO_READ   ((iop->pdat &  CFG_MDIO_PIN) != 0)

#  define MDIO(bit)   if(bit) iop->pdat |=  CFG_MDIO_PIN; \
                        else    iop->pdat &= ~CFG_MDIO_PIN

#  define MDC(bit)    if(bit) iop->pdat |=  CFG_MDC_PIN; \
                        else    iop->pdat &= ~CFG_MDC_PIN
#  define MIIDELAY    udelay(1)

#endif /* defined(CONFIG_ETHER_ON_SCC) || defined(CONFIG_ETHER_ON_FCC) */


/*
 *  PCI config 
 */
#define CONFIG_PCI
#define CONFIG_PCI_PNP
#define CONFIG_PCI_BOOTDELAY 0
#define CONFIG_PCI_SCAN_SHOW

   /* MASTER Window */
#define CFG_PCI_MSTR0_LOCAL         0x80000000        /* Local base */
#define CFG_PCIMSK0_MASK            PCIMSK_1GB        /* Size of window */
#ifdef CONFIG_K26
#define CFG_PCIMSK1_MASK            PCIMSK_32MB	      /* Size of window */
#else
#define CFG_PCIMSK1_MASK            PCIMSK_64MB	      /* Size of window */
#endif

   /* PCI Outbound Window Mem1 */
#define CFG_PCI_MSTR_MEM_LOCAL      0x80000000        /* Local base */
#define CFG_PCI_MSTR_MEM_BUS        0x80000000        /* PCI base   */
#define CFG_CPU_PCI_MEM_START       CFG_PCI_MSTR_MEM_LOCAL
#define CFG_PCI_MSTR_MEM_SIZE       0x20000000        /* 512MB */
#ifdef CONFIG_K26
#  define CFG_POCMR0_MASK_ATTRIB      (POCMR_MASK_512MB | POCMR_ENABLE | POCMR_PREFETCH_EN)
#else
#  define CFG_POCMR0_MASK_ATTRIB      (POCMR_MASK_512MB | POCMR_ENABLE)
#endif

 /* PCI Outbound Window Mem2 */
#define CFG_PCI_MSTR_MEMIO_LOCAL    0xa0000000
#define CFG_PCI_MSTR_MEMIO_BUS      0xa0000000
#define CFG_CPU_PCI_MEMIO_START     PCI_MSTR_MEMIO_LOCAL
#ifdef CONFIG_K26
#  define CFG_PCI_MSTR_MEMIO_SIZE     0x20000000          /* 512MB */
#  define CFG_POCMR1_MASK_ATTRIB      (POCMR_MASK_512MB | POCMR_ENABLE)
#else
#  define CFG_PCI_MSTR_MEMIO_SIZE     0
#  define CFG_POCMR1_MASK_ATTRIB      0
#endif

/* PCI Outbound Window I/O */
#ifdef CONFIG_K26
#  define CFG_PCI_MSTR_IO_BUS         0x00000000          /* PCI base   */
#  define CFG_PCI_MSTR_IO_LOCAL       0xF6000000          /* Local base */
#  define CFG_PCI_MSTR_IO_SIZE        0x02000000          /* 32MB */
#  define CFG_POCMR2_MASK_ATTRIB      (POCMR_MASK_32MB | POCMR_ENABLE | POCMR_PCI_IO)
#else
#  define CFG_PCI_MSTR_IO_BUS         0xa0000000          /* PCI base   */
#  define CFG_PCI_MSTR_IO_LOCAL       0xa0000000          /* Local base */
#  define CFG_PCI_MSTR_IO_SIZE        0x04000000          /* 64MB */
#  define CFG_POCMR2_MASK_ATTRIB      (POCMR_MASK_64MB | POCMR_ENABLE | POCMR_PCI_IO)
#endif
#define CFG_CPU_PCI_IO_START        CFG_PCI_MSTR_IO_LOCAL

 /* PCI Inbound Window */
#define CFG_PCI_SLV_MEM_LOCAL       CFG_SDRAM_BASE /* Local base */
#define CFG_PCI_SLV_MEM_BUS         0x00000000      /* PCI base */
#define CFG_PICMR0_MASK_ATTRIB      (PICMR_MASK_256MB | PICMR_ENABLE | PICMR_PREFETCH_EN)

#ifndef CONFIG_8260_CLKIN
#if CONFIG_ADSTYPE >= CFG_PQ2FADS
#define CONFIG_8260_CLKIN       100000000       /* in Hz */
#else
#define CONFIG_8260_CLKIN       66000000        /* in Hz */
#endif
#endif

#define CONFIG_BAUDRATE         115200

#define CFG_EXCLUDE              CFG_CMD_BEDBUG | \
                                 CFG_CMD_BMP    | \
                                 CFG_CMD_BSP    | \
                                 CFG_CMD_DATE   | \
                                 CFG_CMD_DOC    | \
                                 CFG_CMD_DTT    | \
                                 CFG_CMD_EEPROM | \
                                 CFG_CMD_ELF    | \
                                 CFG_CMD_EXT2   | \
                                 CFG_CMD_FAT    | \
                                 CFG_CMD_FDC    | \
                                 CFG_CMD_FDOS   | \
                                 CFG_CMD_HWFLOW | \
                                 CFG_CMD_IDE    | \
                                 CFG_CMD_KGDB   | \
                                 CFG_CMD_MMC    | \
                                 CFG_CMD_NAND   | \
                                 CFG_CMD_PCMCIA | \
                                 CFG_CMD_REISER | \
                                 CFG_CMD_SCSI   | \
                                 CFG_CMD_SPI    | \
                                 CFG_CMD_UNIVERSE | \
                                 CFG_CMD_USB    | \
                                 CFG_CMD_VFD    | \
                                 CFG_CMD_XIMG

#ifndef CONFIG_MII
#define CONFIG_COMMANDS         (CFG_CMD_ALL & ~( \
                                 CFG_CMD_SDRAM  | \
                                 CFG_CMD_I2C    | \
                                 CFG_CMD_MII | \
                                 CFG_EXCLUDE    ) )
#else
#define CONFIG_COMMANDS         (CFG_CMD_ALL & ~( \
                                 CFG_CMD_SDRAM  | \
                                 CFG_CMD_I2C    | \
                                 CFG_EXCLUDE    ) )
#endif

/* this must be included AFTER the definition of CONFIG_COMMANDS (if any) */
#include <cmd_confdefs.h>

#define CONFIG_BOOTDELAY            0
#define CONFIG_ZERO_BOOTDELAY_CHECK 1
#define CONFIG_ZERO_BOOTDELAY_DIPSW 1
#define CONFIG_BOOTSELECT       "auto" /* auto | normal | recovery */
#ifdef CONFIG_SONYSY355_01
#  define CONFIG_BOOTARGS         "root=/dev/mtdblock1 rootfstype=jffs2 ro console=ttyCPM0,115200"
#  define CONFIG_BOOTARGS_R       "root=/dev/mtdblock3 rootfstype=jffs2 ro console=ttyCPM0,115200 bootmode=recovery"
#  define CONFIG_BOOTARGS_J       "root=/dev/mtdblock9 rootfstype=jffs2 ro console=ttyCPM0,115200 jigboot=yes"
#  define CONFIG_BOOTARGS_JR      "root=/dev/mtdblock11 rootfstype=jffs2 ro console=ttyCPM0,115200 jigboot=yes bootmode=recovery"
#define CONFIG_BOOTCOMMAND      "bootm 0xfc000000"
#define CONFIG_BOOTCOMMAND_R    "bootm 0xfe000000"
#define CONFIG_BOOTCOMMAND_J    "bootm 0xfc000000"
#define CONFIG_BOOTCOMMAND_JR   "bootm 0xfe000000"
#else
#  define CONFIG_BOOTARGS         "root=/dev/ram rw console=ttyCPM0,115200"
#  define CONFIG_BOOTARGS_R       "root=/dev/ram rw console=ttyCPM0,115200 bootmode=recovery"
#  define CONFIG_BOOTARGS_J       "root=/dev/ram rw console=ttyCPM0,115200 jigboot=yes"
#  define CONFIG_BOOTARGS_JR      "root=/dev/ram rw console=ttyCPM0,115200 jigboot=yes bootmode=recovery"
#define CONFIG_BOOTCOMMAND      "bootm fff80000 fff80000 fc000000"
#define CONFIG_BOOTCOMMAND_R    "bootm ff000000 ff200000"
#define CONFIG_BOOTCOMMAND_J    "bootm fff80000 fff80000 fc000000"
#define CONFIG_BOOTCOMMAND_JR   "bootm ff000000 ff200000"
#endif

#define CONFIG_INITRD_HIGH      "7000000"

#define CONFIG_EXTRA_ENV_SETTINGS \
    "bootselect="  CONFIG_BOOTSELECT	 "\0" \
    "bootargsr="   CONFIG_BOOTARGS_R	 "\0" \
    "bootargsj="   CONFIG_BOOTARGS_J	 "\0" \
    "bootargsjr="  CONFIG_BOOTARGS_JR	 "\0" \
    "bootcmdr="	   CONFIG_BOOTCOMMAND_R	 "\0" \
    "bootcmdj="	   CONFIG_BOOTCOMMAND_J	 "\0" \
    "bootcmdjr="   CONFIG_BOOTCOMMAND_JR "\0" \
    "initrd_high=" CONFIG_INITRD_HIGH	 "\0"

#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#undef  CONFIG_KGDB_ON_SMC              /* define if kgdb on SMC */
#define CONFIG_KGDB_ON_SCC              /* define if kgdb on SCC */
#undef  CONFIG_KGDB_NONE                /* define if kgdb on something else */
#define CONFIG_KGDB_INDEX       2       /* which serial channel for kgdb */
#define CONFIG_KGDB_BAUDRATE    115200  /* speed to run kgdb serial port at */
#endif

#define CONFIG_BZIP2    /* include support for bzip2 compressed images */
#undef  CONFIG_WATCHDOG /* disable platform specific watchdog */


/*
 * Miscellaneous configurable options
 */
#define CFG_HUSH_PARSER
#define CFG_PROMPT_HUSH_PS2 "> "
#define CFG_LONGHELP                    /* undef to save memory     */
#define CFG_PROMPT      "sy355=> "       /* Monitor Command Prompt   */
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CFG_CBSIZE      1024            /* Console I/O Buffer Size  */
#else
#define CFG_CBSIZE      256                     /* Console I/O Buffer Size  */
#endif
#define CFG_PBSIZE (CFG_CBSIZE+sizeof(CFG_PROMPT)+16)   /* Print Buffer Size */
#define CFG_MAXARGS     16                      /* max number of command args   */
#define CFG_BARGSIZE    CFG_CBSIZE      /* Boot Argument Buffer Size    */

#define CFG_DRAM_TEST				/* memory test, takes time */
#define CFG_MEMTEST_START       0x00100000      /* memtest works on */
#define CFG_MEMTEST_END         0x00f00000      /* 1 ... 15 MB in DRAM  */
#define CFG_MEMTEST_ALL_START	0x00000000	/* memory all test */
#define CFG_MEMTEST_ALL_END		0x10000000

#define CFG_LOAD_ADDR           0x100000        /* default load address */

#define CFG_HZ                  1000    /* decrementer freq: 1 ms ticks */

#define CFG_BAUDRATE_TABLE      { 9600, 19200, 38400, 57600, 115200, 230400 }

/* Flash bank, timeout, e.t.c.  */
#ifdef CONFIG_SY355_01
#  define CFG_MAX_FLASH_BANKS     4       /* max num of memory banks      */
#  define CFG_MAX_FLASH_SECT      128     /* max num of sects on one chip */
#  define CFG_FLASH_SIZE          32      /* 32MB */
#else
#  define CFG_MAX_FLASH_BANKS     2       /* max num of memory banks      */
#  define CFG_MAX_FLASH_SECT      259     /* max num of sects on one chip */
#  define CFG_FLASH_SIZE          64      /* 64MB */
#endif
#define CFG_FLASH_LOCK_TOUT     5       /* Timeout for Flash Set Lock Bit (in ms) */
#define CFG_FLASH_UNLOCK_TOUT   10000   /* Timeout for Flash Clear Lock Bits (in ms) */
#undef  CFG_FLASH_CHECKSUM
#define CFG_FLASH_ERASE_TOUT    60000   /* Timeout for Flash Erase (in ms)*/
#define CFG_FLASH_WRITE_TOUT    1000    /* Timeout for Flash Write (in ms)*/

#define CFG_FLASH_CFI           1


/* this is stuff came out of the Motorola docs */
#define CFG_DEFAULT_IMMR        0x0F010000
#define CFG_IMMR                0xF0000000
#define CFG_SDRAM_BASE          0x00000000

#define CFG_JFFS2_FIRST_SECTOR  64
#define CFG_JFFS2_LAST_SECTOR   127
#define CFG_JFFS2_SORT_FRAGMENTS
#define CFG_JFFS_CUSTOM_PART

#define RS232EN_1               0x02000002
#define RS232EN_2               0x01000001
#define FETHIEN1                0x08000008
#define FETH1_RST               0x04000004
#define FETHIEN2                0x10000000
#define FETH2_RST               0x08000000

#define CFG_INIT_RAM_ADDR       CFG_IMMR
#define CFG_INIT_RAM_END        0x2000  /* End of used area in DPRAM    */
#define CFG_GBL_DATA_SIZE       128     /* size in bytes reserved for initial data */
#define CFG_GBL_DATA_OFFSET     (CFG_INIT_RAM_END - CFG_GBL_DATA_SIZE)
#define CFG_INIT_SP_OFFSET      CFG_GBL_DATA_OFFSET


/* 0x0EA28205 */
#define CFG_HRCW_MASTER (   ( HRCW_BPS11 | HRCW_CIP )                       |\
                            ( HRCW_L2CPC10 | HRCW_DPPC10 | HRCW_ISB010 )    |\
                            ( HRCW_BMS | HRCW_APPC10 )                      |\
                            ( HRCW_MODCK_H0101 )                             \
                        )
/* no slaves */
#define CFG_HRCW_SLAVE1 0
#define CFG_HRCW_SLAVE2 0
#define CFG_HRCW_SLAVE3 0
#define CFG_HRCW_SLAVE4 0
#define CFG_HRCW_SLAVE5 0
#define CFG_HRCW_SLAVE6 0
#define CFG_HRCW_SLAVE7 0

#define BOOTFLAG_COLD   0x01    /* Normal Power-On: Boot from FLASH  */
#define BOOTFLAG_WARM   0x02    /* Software reboot           */

#define CFG_MONITOR_BASE TEXT_BASE
#if (CFG_MONITOR_BASE < CFG_FLASH_0_BASE)
#   define CFG_RAMBOOT
#endif

#define CFG_MONITOR_LEN         (256 << 10)     /* Reserve 256 kB for Monitor   */
#define CFG_BOOTMAPSZ           (8 << 20)       /* Initial Memory map for Linux */

#ifdef CONFIG_BZIP2
#define CFG_MALLOC_LEN          (4096 << 10)    /* Reserve 4 MB for malloc()    */
#else
#define CFG_MALLOC_LEN          (128 << 10)     /* Reserve 128 KB for malloc()  */
#endif /* CONFIG_BZIP2 */

#ifndef CFG_RAMBOOT
#  ifdef CONFIG_SY355_01
#    define CFG_ENV_IS_IN_FLASH   1
#    define CFG_ENV_SECT_SIZE     0x40000
#    define CFG_ENV_ADDR          0xfff40000
#  else
#    define CFG_ENV_IS_IN_FLASH   1
#    define CFG_ENV_SECT_SIZE     0x10000
#    define CFG_ENV_ADDR          0xfffd0000
#  endif
#else
#  define CFG_ENV_IS_IN_NVRAM   1
#  define CFG_ENV_SIZE          0x200
#  define CFG_ENV_ADDR          (CFG_MONITOR_BASE - 0x1000)
#endif /* CFG_RAMBOOT */

#define CFG_CACHELINE_SIZE      32      /* For MPC8260 CPU */
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#  define CFG_CACHELINE_SHIFT   5       /* log base 2 of the above value */
#endif

#define CFG_HID0_INIT           0
#define CFG_HID0_FINAL          (HID0_ICE | HID0_IFEM | HID0_ABE | HID0_DPM)
#define CFG_HID2                0

#define CFG_SYPCR               0xFFFFFFC3
#define CFG_BCR                 0x100C0000
#define CFG_SIUMCR              0x0D20C000

#ifdef CONFIG_USE_ICE   
#  define CFG_SCCR                SCCR_DFBRG01 /* If JTAG ICE connect */
#else
#  define CFG_SCCR                (SCCR_DFBRG01 | SCCR_CLPD)
#endif


/* Network */
#define CONFIG_BOOTFILE         vmlinux2.img.gz.u-boot
#define CONFIG_SERVERIP         192.168.1.1
#define CONFIG_IPADDR           192.168.1.11
#define CONFIG_GATEWAYIP        192.168.1.254
#define CONFIG_NETMASK          255.255.255.0

/* GPIO */
#define CONFIG_GPIO_A_PN            0
#define CONFIG_GPIO_B_PN            1
#define CONFIG_GPIO_C_PN            2
#define CONFIG_GPIO_D_PN            3

#define CONFIG_GPIO_PN              CONFIG_GPIO_C_PN
#define CONFIG_DIPSW_PN             CONFIG_GPIO_D_PN

#define CONFIG_OUTP_FLASH_PROTECT   0x80000000 /* outport mask (GPIO_PN) */
#define CONFIG_PWR_BTN_X            0x01000000 /* inport mask  (GPIO_PN) */
#define CONFIG_OUTP_POWER_OK        0x00800000 /* outport mask (GPIO_PN) */ 
#define CONFIG_SELF_NRESET          0x00200000 /* outport mask (GPIO_PN) */
#define CONFIG_RECEV_MODE_X         0x00080000 /* inport mask  (GPIO_PN) */
#define CONFIG_JIG_EXIST_X          0x00020000 /* inport mask  (GPIO_PN) */
#define CONFIG_ETHERNET_RST_X       0x00000800 /* inport mask  (GPIO_C_PN) */

#define CONFIG_GPIO_DIPSW_8         0x00002000 /* inport mask (DIPSW_PN) */
#define CONFIG_GPIO_DIPSW_7         0x00001000 /* inport mask (DIPSW_PN) */
#define CONFIG_GPIO_DIPSW_6         0x00000800 /* inport mask (DIPSW_PN) */
#define CONFIG_GPIO_DIPSW_5         0x00000400 /* inport mask (DIPSW_PN) */
#define CONFIG_GPIO_DIPSW_4         0x00000200 /* inport mask (DIPSW_PN) */
#define CONFIG_GPIO_DIPSW_3         0x00000100 /* inport mask (DIPSW_PN) */
#define CONFIG_GPIO_DIPSW_2         0x00000080 /* inport mask (DIPSW_PN) */
#define CONFIG_GPIO_DIPSW_1         0x00000040 /* inport mask (DIPSW_PN) */

#define CONFIG_DIPSW_ZERO_BOOTDELAY CONFIG_GPIO_DIPSW_8
#define CONFIG_DIPSW_DBG_MODE       CONFIG_GPIO_DIPSW_7
#ifndef CONFIG_SONYSY355
#define CONFIG_DIPSW_PWRCTRL        CONFIG_GPIO_DIPSW_5
#else	/* CONFIG_SONYSY355 */
#define CONFIG_DIPSW_RAMTEST		CONFIG_GPIO_DIPSW_5
#endif
#define CONFIG_DIPSW_RECOVERY       CONFIG_GPIO_DIPSW_4
#define CONFIG_DIPSW_NORMAL         CONFIG_GPIO_DIPSW_3
#define CONFIG_DIPSW_BOOTSEL_MASK   (CONFIG_DIPSW_NORMAL | CONFIG_DIPSW_RECOVERY)
#define CONFIG_DIPSW_AUTO           CONFIG_DIPSW_BOOTSEL_MASK   
#define CONFIG_JIG_ROM_ON_X         CONFIG_GPIO_DIPSW_2


#define CONFIG_IBLOADER_BASE 0xfff80000
#ifdef CONFIG_SY355_01
#  define CONFIG_IBLOADER_SIZE 0x00040000
#else
#  define CONFIG_IBLOADER_SIZE 0x00040000
#endif

#ifndef CONFIG_SY355_01
#define CONFIG_IBCONFIG_BASE 0xfffe0000
#define CONFIG_IBCONFIG_SIZE 0x00010000
#define CONFIG_IBCONFIG_DATA "0422"
#define CONFIG_IBCONFIG_DATA_SIZE 4
#endif

#define CONFIG_BOARD_UNIQUE_DATA
#ifdef CONFIG_SY355_01
#  define CFG_BUDATA_ADDR 0xfff80000
#  define CFG_BUDATA_SECTOR_SIZE 0x40000
#  define CFG_BUDATA_SIZE 256 /* byte */
#else
#  define CFG_BUDATA_ADDR 0xfffc0000
#  define CFG_BUDATA_SECTOR_SIZE 0x10000
#  define CFG_BUDATA_SIZE 256 /* byte */
#endif
#define CFG_MAX_ETHDEV 1

#define CONFIG_FLASH_WP_CTRL 1

#endif /* __CONFIG_H */
